@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: callbacks.c,v 1.38 2005/12/04 20:18:49 bchiara Exp $
+ * $Id$
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#include <gnome.h>
+#include "globals.h"
 #include "callbacks.h"
 #include "diagram.h"
 
@@ -34,7 +36,7 @@ on_canvas1_size_allocate (GtkWidget * widget,
 				  -widget->allocation.height / 2,
 				  widget->allocation.width / 2,
 				  widget->allocation.height / 2);
-  need_reposition = TRUE;
+  ask_reposition(FALSE);
   canvas = glade_xml_get_widget (xml, "canvas1");
   update_diagram (canvas);
 }
