@@ -1,6 +1,7 @@
+
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: capture.c,v 1.166 2006/08/13 05:26:50 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,16 +45,17 @@ typedef enum
 create_node_type_t;
 
 static pcap_t *pch_struct;		/* pcap structure */
-static struct pcap_pkthdr phdr;
 static gint pcap_fd;			/* The file descriptor used by libpcap */
 static gint capture_source;		/* It's the input tag or the timeout tag,
 				 * in online or offline mode */
-static guint32 ms_to_next;	/* Used for offline mode to store the amount
+static guint ms_to_next;	/* Used for offline mode to store the amount
 				 * of time that we have to wait between
 				 * one packet and the next */
 static guint node_id_length;		/* Length of the node_id key. Depends
 				 * on the mode of operation */
-
+static guint l3_offset;		/* Offset to the level 3 protocol data
+				 * Depends of the linktype */
+static enum status_t capture_status = STOP;
 
 /* Local funtions declarations */
 static guint get_offline_packet (void);
@@ -68,10 +70,7 @@ static void add_node_packet (const guint8 * packet,
                              const node_id_t *node_id,
 			     packet_direction direction);
 
-static void set_node_name (node_t * node, const gchar * preferences);
 
-
-static GString *print_mem (const node_id_t *node_id);
 static void dns_ready (gpointer data, gint fd, GdkInputCondition cond);
 
 
@@ -162,6 +161,11 @@ get_node_id (const guint8 * raw_packet, size_t raw_size, create_node_type_t node
   return node_id;
 }				/* get_node_id */
 
+enum status_t get_capture_status(void)
+{
+  return capture_status;
+}
+
 /* Sets up the pcap device
  * Sets up the mode and related variables
  * Sets up dns if needed
@@ -170,7 +174,6 @@ get_node_id (const guint8 * raw_packet, size_t raw_size, create_node_type_t node
 gchar *
 init_capture (void)
 {
-
   gchar *device;
   gchar ebuf[300];
   gchar *str = NULL;
@@ -178,7 +181,6 @@ init_capture (void)
   static gchar errorbuf[300];
   static gboolean data_initialized = FALSE;
 
-
   if (!data_initialized)
     {
 
@@ -193,10 +195,8 @@ init_capture (void)
              }
 	}
 
-      status = STOP;
+      capture_status = STOP;
       data_initialized = TRUE;
-
-      n_packets = total_mem_packets = 0;
     }
 
   device = pref.interface;
@@ -217,7 +217,6 @@ init_capture (void)
     }
 
 
-  end_of_file = FALSE;
   if (!pref.input_file)
     {
       *ebuf = '\0'; /* reset error buffer before calling pcap functions */
@@ -335,7 +334,7 @@ init_capture (void)
 	}
       break;
     case TCP:
-      if (pref.filter)
+      if (pref.filter && *pref.filter)
 	str = g_strconcat ("tcp and ", pref.filter, NULL);
       else
 	{
@@ -344,19 +343,8 @@ init_capture (void)
 	  str = g_strdup ("tcp");
 	}
       break;
-    case UDP:
-      if (pref.filter)
-	str = g_strconcat ("udp and ", pref.filter, NULL);
-      else
-	{
-	  g_free (pref.filter);
-	  pref.filter = NULL;
-	  str = g_strdup ("udp");
-	}
-      break;
     case DEFAULT:
     case ETHERNET:
-    case IPX:
     case FDDI:
     case IEEE802:
       if (pref.filter)
@@ -371,7 +359,6 @@ init_capture (void)
   if (pref.filter)
     set_filter (pref.filter, device);
 
-
   if (error)
     {
       snprintf (errorbuf, sizeof(errorbuf), _("Mode not available in this device"));
@@ -400,7 +387,6 @@ init_capture (void)
       return errorbuf;
     }
 
-
   return NULL;
 }				/* init_capture */
 
@@ -431,26 +417,6 @@ set_filter (gchar * filter_string, gchar * device)
     g_warning (_("Can't install filter (%s)."), pcap_geterr (pch_struct));
 
   return 0;
-#if 0
-  /* TODO pending to be more general, since we want to be able 
-   * to change the capturing device in runtime. */
-  if (!current_device)
-    current_device = g_strdup (device);
-
-  /* A capture filter was specified; set it up. */
-  if (current_device
-      && (pcap_lookupnet (current_device, &netnum, &netmask, ebuf) < 0))
-    {
-      g_warning (_
-		 ("Couldn't obtain netmask info (%s). Filters involving broadcast addresses could behave incorrectly."),
-		 ebuf);
-      netmask = 0;
-    }
-  if (pcap_compile (pch_struct, &fp, filter_string, 1, netmask) < 0)
-    g_warning (_("Unable to parse filter string (%s)."), pcap_geterr (pch_struct));
-  else if (pcap_setfilter (pch_struct, &fp) < 0)
-    g_warning (_("Can't install filter (%s)."), pcap_geterr (pch_struct));
-#endif
 }				/* set_filter */
 
 gboolean
@@ -458,14 +424,11 @@ start_capture (void)
 {
   GnomeCanvas *gc;
 
-  if ((status != PAUSE) && (status != STOP))
-    {
-      g_warning (_("Status not PAUSE or STOP at start_capture"));
-      return FALSE;
-    }
+  if (capture_status == PLAY)
+    return TRUE;
 
   /* if it's a new capture, we prepare protocol summary and nodes/links catalogs */
-  if (STOP == status)
+  if (STOP == capture_status)
     {
       protocol_summary_open();
       nodes_catalog_open();
@@ -476,7 +439,7 @@ start_capture (void)
    * See pause_capture for an explanation of why we don't always
    * add the source
    */
-  if (pref.interface && (status == STOP))
+  if (pref.interface && (capture_status == STOP))
     {
       g_my_debug (_("Starting live capture"));
       capture_source = gdk_input_add (pcap_fd,
@@ -496,45 +459,29 @@ start_capture (void)
   /* set the antialiasing */
   gc = GNOME_CANVAS (glade_xml_get_widget (xml, "canvas1"));
   if (gc)
-    gc->aa = pref.antialias;
+    gc->aa = TRUE;
 
-  status = PLAY;
+  capture_status = PLAY;
   return TRUE;
 }				/* start_capture */
 
 gboolean
 pause_capture (void)
 {
-  if (status != PLAY)
-    g_warning (_("Status not PLAY at pause_capture"));
+  if (capture_status != PLAY)
+    return TRUE;
 
-  if (pref.interface)
-    {
-      /* Why would we want to miss packets while pausing to 
-       * better analyze a moment in time? If we wanted
-       * to do this it should be optional at least.
-       * In order for this to work, start_capture should only
-       * add the source if the pause was for an offline capture */
-#if 0
-      g_my_debug (_("Pausing live capture"));
-      gdk_input_remove (capture_source);	/* gdk_input_remove does not
-						 * return an error code */
-#endif
-    }
-  else
+  if (!pref.interface)
     {
       g_my_debug (_("Pausing offline capture"));
-      if (!end_of_file)
-	{
-	  if (!g_source_remove (capture_source))
-	    {
-	      g_warning (_("Error while trying to pause capture"));
-	      return FALSE;
-	    }
-	}
+      if (!g_source_remove (capture_source))
+        {
+          g_warning (_("Error while trying to pause capture"));
+          return FALSE;
+        }
     }
 
-  status = PAUSE;
+  capture_status = PAUSE;
   return TRUE;
 
 }
@@ -544,12 +491,8 @@ gboolean
 stop_capture (void)
 {
   struct pcap_stat ps;
-
-  if ((status != PLAY) && (status != PAUSE))
-    {
-      g_warning (_("Status not PLAY or PAUSE at stop_capture"));
-      return FALSE;
-    }
+  if (capture_status == STOP)
+      return TRUE;
 
   if (pref.interface)
     {
@@ -558,20 +501,9 @@ stop_capture (void)
 						 * return an error code */
     }
   else
-    {
       g_my_debug (_("Stopping offline capture"));
-      if (!end_of_file)
-	{
-	  if (!g_source_remove (capture_source))
-	    {
-	      g_warning (_
-			 ("Error while removing capture source in stop_capture"));
-	      return FALSE;
-	    }
-	}
-    }
 
-  status = STOP;
+  capture_status = STOP;
 
   /* free nodes, protocols, links, conversations and packets */
   protocol_summary_close();
@@ -596,11 +528,9 @@ stop_capture (void)
   pcap_stats (pch_struct, &ps);
   g_my_debug ("libpcap received %d packets, dropped %d. EtherApe saw %g",
 	      ps.ps_recv, ps.ps_drop, n_packets);
-  n_packets = 0;
   pcap_close (pch_struct);
   g_my_info (_("Capture device stopped or file closed"));
 
-  
   return TRUE;
 }				/* stop_capture */
 
@@ -611,8 +541,7 @@ stop_capture (void)
 void
 cleanup_capture (void)
 {
-  if (status != STOP)
-    stop_capture ();
+  stop_capture ();
   dns_close(); /* closes the dns resolver, if opened */
 }
 
@@ -623,49 +552,57 @@ cleanup_capture (void)
 static guint
 get_offline_packet (void)
 {
-  static guint8 *packet = NULL;
+  static struct pcap_pkthdr *pkt_header = NULL;
+  static const u_char *pkt_data = NULL;
   static struct timeval last_time = { 0, 0 }, this_time, diff;
+  int result;
 
-  if (status == STOP)
+  if (capture_status == STOP)
     {
-      packet = NULL;
+      pkt_header = NULL;
+      pkt_data = NULL;
       last_time.tv_usec = last_time.tv_sec = 0;
       return FALSE;
     }
 
-  if (packet)
+  if (pkt_data)
   {
     gettimeofday (&now, NULL);
-    packet_acquired(packet, phdr.caplen, phdr.len);
+    packet_acquired( (guint8 *)pkt_data, pkt_header->caplen, pkt_header->len);
   }
 
-  packet = (guint8 *) pcap_next (pch_struct, &phdr);
-  if (!packet)
-    end_of_file = TRUE;
-
-  if (last_time.tv_sec == 0 && last_time.tv_usec == 0)
+  result = pcap_next_ex(pch_struct, &pkt_header, &pkt_data);
+  switch (result)
     {
-      last_time.tv_sec = phdr.ts.tv_sec;
-      last_time.tv_usec = phdr.ts.tv_usec;
+    case 1:
+      if (last_time.tv_sec == 0 && last_time.tv_usec == 0)
+        {
+          last_time.tv_sec = pkt_header->ts.tv_sec;
+          last_time.tv_usec = pkt_header->ts.tv_usec;
+        }
+
+      this_time.tv_sec = pkt_header->ts.tv_sec;
+      this_time.tv_usec = pkt_header->ts.tv_usec;
+
+      diff = substract_times (this_time, last_time);
+
+      /* diff can be negative when listening to multiple interfaces.
+       * In that case the delay is zeroed */
+      if (diff.tv_sec < 0)
+        ms_to_next = 0; 
+      else 
+        ms_to_next = diff.tv_sec * 1000 + diff.tv_usec / 1000;
+      if (ms_to_next > pref.max_delay)
+          ms_to_next = pref.max_delay;
+
+      last_time = this_time;
+      break;
+    case -2:
+      capture_status = CAP_EOF;
+      break;
+    default:
+      ms_to_next=0; /* error or timeout, ignore packet */
     }
-
-  this_time.tv_sec = phdr.ts.tv_sec;
-  this_time.tv_usec = phdr.ts.tv_usec;
-
-  diff = substract_times (this_time, last_time);
-
-  if (pref.zero_delay)
-    {
-      ms_to_next = 0;
-    }
-  else
-    {
-      ms_to_next = diff.tv_sec * 1000 + diff.tv_usec / 1000;
-    }
-
-
-  last_time = this_time;
-
   return FALSE;
 }				/* get_offline_packet */
 
@@ -674,34 +611,35 @@ static void
 cap_t_o_destroy (gpointer data)
 {
 
-  if ((status == PLAY) && !end_of_file)
+  if (capture_status == PLAY)
     capture_source = g_timeout_add_full (G_PRIORITY_DEFAULT,
 					 ms_to_next,
 					 (GtkFunction) get_offline_packet,
 					 data,
 					 (GDestroyNotify) cap_t_o_destroy);
 
-}				/* capture_t_o_destroy */
+}				/* cap_t_o_destroy */
 
 
 /* This function is the gdk callback called when the capture socket holds data */
 static void
 read_packet_live(gpointer dummy, gint source, GdkInputCondition condition)
 {
-  guint8 * packet = NULL;
+  struct pcap_pkthdr *pkt_header = NULL;
+  const u_char *pkt_data = NULL;
+  int result;
 
   /* Get next packet */
-  packet = (guint8 *) pcap_next (pch_struct, &phdr);
+  result = pcap_next_ex(pch_struct, &pkt_header, &pkt_data);
 
-  /* Redhat's phdr.ts is not a timeval, so I can't
+  /* Redhat's pkt_header.ts is not a timeval, so I can't
    * just copy the structures */
-  now.tv_sec = phdr.ts.tv_sec;
-  now.tv_usec = phdr.ts.tv_usec;
-
-  if (packet)
-    packet_acquired(packet, phdr.caplen, phdr.len);
-
-}				/* packet_read */
+  now.tv_sec = pkt_header->ts.tv_sec;
+  now.tv_usec = pkt_header->ts.tv_usec;
+ 
+  if (pkt_data)
+    packet_acquired( (guint8 *)pkt_data, pkt_header->caplen, pkt_header->len);
+}
 
 /* This function is called everytime there is a new packet in
  * the network interface. It then updates traffic information
@@ -711,23 +649,22 @@ static void
 packet_acquired(guint8 * raw_packet, guint raw_size, guint pkt_size)
 {
   packet_info_t *packet;
-  const gchar *prot_desc = NULL;
   node_id_t src_node_id;
   node_id_t dst_node_id;
   link_id_t link_id;
 
-  /* Get a string with the protocol tree */
-  prot_desc = get_packet_prot (raw_packet, raw_size);
-
   /* We create a packet structure to hold data */
   packet = g_malloc (sizeof (packet_info_t));
+  g_assert(packet);
+  
   packet->size = pkt_size;
   packet->timestamp = now;
-  packet->prot_desc = g_strdup (prot_desc);
   packet->ref_count = 0;
 
-  /* If there is no node with that id, create it. Otherwise 
-   * just use the one available */
+  /* Get a string with the protocol tree */
+  packet->prot_desc = get_packet_prot (raw_packet, raw_size, 
+                                       linktype, l3_offset);
+
   src_node_id = get_node_id (raw_packet, raw_size, SRC);
   dst_node_id = get_node_id (raw_packet, raw_size, DST);
 
@@ -765,14 +702,8 @@ add_node_packet (const guint8 * raw_packet,
   if (node == NULL)
     {
       /* creates the new node, adding it to the catalog */
-      GString *node_id_str = print_mem (node_id);
-      node = node_create(node_id, node_id_str->str);
-      nodes_catalog_insert(node);
-    
-      g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-             _("Creating node: %s. Number of nodes %d"),
-             node_id_str->str, nodes_catalog_size());
-      g_string_free(node_id_str, TRUE);
+      node = nodes_catalog_new(node_id);
+      g_assert(node);
     }
 
   traffic_stats_add_packet(&node->node_stats, packet, direction);
@@ -796,166 +727,3 @@ dns_ready (gpointer data, gint fd, GdkInputCondition cond)
   dns_ack ();
 }
 
-/* Sets the node->name and node->numeric_name to the most used of 
- * the default name for the current mode */
-void
-update_node_names (node_t * node)
-{
-  GList *protocol_item;
-  protocol_t *protocol;
-  guint i = STACK_SIZE;
-
-  /* TODO Check if it's while i or while i+1. 
-   * Then fix it in other places */
-  while (i + 1)
-    {
-      for ( protocol_item = node->node_stats.stats_protos.protostack[i]; 
-            protocol_item; 
-            protocol_item = protocol_item->next)
-        {
-          protocol = (protocol_t *) (protocol_item->data);
-          protocol->node_names
-            = g_list_sort (protocol->node_names, node_name_freq_compare);
-        }
-
-      i--;
-    }
-
-  switch (pref.mode)
-    {
-    case ETHERNET:
-      set_node_name (node,
-		     "ETH_II,SOLVED;802.2,SOLVED;803.3,SOLVED;"
-		     "NETBIOS-DGM,n;NETBIOS-SSN,n;IP,n;"
-		     "IPX-SAP,n;ARP,n;" "ETH_II,n;802.2,n;802.3,n");
-      break;
-    case FDDI:
-      set_node_name (node,
-		     "FDDI,SOLVED;NETBIOS-DGM,n;NETBIOS-SSN,n;IP,n;ARP,n;FDDI,n");
-      break;
-    case IEEE802:
-      set_node_name (node,
-		     "IEEE802,SOLVED;NETBIOS-DGM,n;NETBIOS-SSN,n;IP,n;ARP,n;IEEE802,n");
-      break;
-    case IP:
-      set_node_name (node, "NETBIOS-DGM,n;NETBIOS-SSN,n;IP,n");
-      break;
-    case TCP:
-      set_node_name (node, "TCP,n");
-      break;
-    default:
-      break;
-    }
-}				/* update_node_names */
-
-
-static void
-set_node_name (node_t * node, const gchar * preferences)
-{
-  GList *name_item = NULL;
-  name_t *name = NULL;
-  const protocol_t *protocol = NULL;
-  gchar **prots, **tokens;
-  guint i = 0;
-  guint j = STACK_SIZE;
-  gboolean cont = TRUE;
-
-  prots = g_strsplit (preferences, ";", 0);
-  for (; prots[i] && cont; i++)
-    {
-      tokens = g_strsplit (prots[i], ",", 0);
-
-      /* We don't do level 0, which has the topmost prot */
-      for (j = STACK_SIZE; j && cont; j--)
-	{
-	  protocol = protocol_stack_find(&node->node_stats.stats_protos, j, tokens[0]);
-	  if (protocol)
-	    {
-	      name_item = protocol->node_names;
-	      if (!strcmp (protocol->name, tokens[0]) && name_item)
-		{
-		  name = (name_t *) (name_item->data);
-		  /* If we require this protocol to be solved and it's not,
-		   * the we have to go on */
-		  if (strcmp (tokens[1], "SOLVED") || name->solved)
-		    {
-		      if (node->name && strcmp (node->name->str, name->name->str))
-			  g_my_debug ("Switching node name from %s to %s",
-				      node->name->str, name->name->str);
-
-		      g_string_assign (node->name, name->name->str);
-		      g_string_assign (node->numeric_name,
-				       name->numeric_name->str);
-		      cont = FALSE;
-		    }
-		}
-	    }
-
-	}
-      g_strfreev (tokens);
-      tokens = NULL;
-    }
-  g_strfreev (prots);
-  prots = NULL;
-}				/* set_node_name */
-
-
-/* creates a new string from the given address */
-static GString *
-print_mem (const node_id_t *node_id)
-{
-  static const gchar hex_digits[16] = "0123456789abcdef";
-  gchar cur[50];
-  char punct = ':';
-  gchar *p;
-  int i;
-  guint32 octet;
-  const guint8 *ad = NULL;
-  guint length = 0;
-
-  switch (node_id->node_type)
-    {
-    case ETHERNET:
-      ad = node_id->addr.eth;
-      length = sizeof(node_id->addr.eth);
-      break;
-    case FDDI:
-      ad = node_id->addr.fddi;
-      length = sizeof(node_id->addr.fddi);
-      break;
-    case IEEE802:
-      ad = node_id->addr.i802;
-      length = sizeof(node_id->addr.i802);
-      break;
-    case IP:
-      ad = node_id->addr.ip4;
-      length = sizeof(node_id->addr.ip4);
-      break;
-    case TCP:
-      ad = node_id->addr.tcp4.host;
-      length = sizeof(node_id->addr.tcp4);
-      break;
-    default:
-      g_error (_("Unsopported ape mode in print_mem"));
-    }
-
-  if (length * 3 >= 50)
-     g_error (_("Invalid length in print_mem"));
-    
-  p = &cur[length * 3];
-  *--p = '\0';
-  i = length - 1;
-  for (;;)
-    {
-      octet = ad[i];
-      *--p = hex_digits[octet & 0xF];
-      octet >>= 4;
-      *--p = hex_digits[octet & 0xF];
-      if (i == 0)
-	break;
-      if (punct)
-	*--p = punct;
-      i--;
-    }
-  return g_string_new(p);
-}				/* print_mem */
