@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: capture.h,v 1.61 2005/12/30 16:36:09 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,19 @@
 #ifndef ETHERAPE_CAPTURE_H
 #define ETHERAPE_CAPTURE_H
 
+#include "links.h"
+
+/* Possible states of capture status */
+enum status_t
+{
+  STOP = 0, 
+  PLAY = 1, 
+  PAUSE = 2,
+  CAP_EOF = 3 /* end-of-file */
+};
+
+enum status_t get_capture_status(void);
+
 gchar *init_capture (void);
 gboolean start_capture (void);
 gboolean pause_capture (void);
@@ -26,6 +39,4 @@ gboolean stop_capture (void);
 void cleanup_capture (void);
 gint set_filter (gchar * filter, gchar * device);
 
-void update_node_names (node_t * node);
-
 #endif
