@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: conversations.c,v 1.6 2006/01/26 07:18:59 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,15 @@
 #include "conversations.h"
 #include "dns.h"
 
-static GList *conversations = NULL;	/* Some protocols add an item here to help identify
-				 * further packets of the same protocol */
+/* Some protocols add an item here to help identify further packets of the 
+ * same protocol */
+static GList *conversations = NULL;
+static long n_conversations = 0;
+
+long active_conversations(void)
+{
+  return n_conversations;
+}
 
 /* Returns the item ptr if there is any matching conversation in any of the
  * two directions */
@@ -70,7 +77,7 @@ add_conversation (guint32 src_address, guint32 dst_address,
 		  guint16 src_port, guint16 dst_port, const gchar * data)
 {
   conversation_t *conv = NULL;
-  gchar *old_data = NULL;
+  const gchar *old_data = NULL;
   guint32 src, dst;
 
   /* Because that is the way that ip_to_str works */
@@ -99,6 +106,8 @@ add_conversation (guint32 src_address, guint32 dst_address,
 	      ip_to_str ((guint8 *) & dst), dst_port, data);
 
   conv = g_malloc (sizeof (conversation_t));
+  g_assert(conv);
+  
   conv->src_address = src_address;
   conv->dst_address = dst_address;
   conv->src_port = src_port;
@@ -106,14 +115,14 @@ add_conversation (guint32 src_address, guint32 dst_address,
   conv->data = g_strdup (data);
 
   conversations = g_list_prepend (conversations, conv);
-
+  n_conversations++;
 }				/* add_conversation */
 
 
 /* Returns the data if there is any matching conversation in any of the
  * two directions */
 /* A zero in any of the ports matches any port number */
-gchar* 
+const gchar* 
 find_conversation (guint32 src_address, guint32 dst_address,
 		       guint16 src_port, guint16 dst_port)
 {
@@ -151,6 +160,7 @@ delete_conversation_link(guint32 src_address, guint32 dst_address)
       g_free (conv->data);
       g_free (conv);
       conversations = g_list_delete_link(conversations, item);
+      n_conversations--;
     }
 }
 
@@ -173,6 +183,7 @@ delete_conversations (void)
       g_free (conv->data);
       g_free (conv);
       item = item->next;
+      n_conversations--;
     }
 
   g_list_free (conversations);
