@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: conversations.h,v 1.4 2006/01/26 07:18:59 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,9 @@ conversation_t;
 
 void add_conversation (guint32 src_address, guint32 dst_address,
 		       guint16 src_port, guint16 dst_port, const gchar * data);
-gchar *find_conversation (guint32 src_address, guint32 dst_address,
+const gchar *find_conversation (guint32 src_address, guint32 dst_address,
 			  guint16 src_port, guint16 dst_port);
 void delete_conversation_link(guint32 src_address, guint32 dst_address);
 void delete_conversations (void);
+
+long active_conversations(void);
