@@ -1,7 +1,7 @@
 
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: decode_proto.h,v 1.3 2006/01/26 07:18:59 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,12 @@
 #ifndef DECODE_PROTO_H
 #define DECODE_PROTO_H
 
-/* extracts the protocol stack from packet, and returs it as string */
-const gchar *get_packet_prot (const guint8 * packet, guint raw_size);
+#include "pkt_info.h"
+
+/* extracts the protocol stack from packet, and returs it as a newly allocated
+ * packet_protos_t */
+packet_protos_t *get_packet_prot (const guint8 * packet, guint raw_size, 
+                                  link_type_t link_type, guint l3_offset);
 
 
 #endif
