@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: diagram.c,v 1.150 2006/09/30 05:57:11 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,8 +23,6 @@
 
 #include <gnome.h>
 
-#include "math.h"
-
 #include "globals.h"
 #include "diagram.h"
 #include "preferences.h"
@@ -32,11 +30,13 @@
 #include "info_windows.h"
 #include "protocols.h"
 #include "datastructs.h"
+#include "capture.h"
 
 /* maximum node and link size */
 #define MAX_NODE_SIZE 5000
 #define MAX_LINK_SIZE (MAX_NODE_SIZE/4)
 
+
 /***************************************************************************
  *
  * canvas_node_t definition and methods
@@ -88,11 +88,11 @@ static gint canvas_link_update(link_id_t * link_id,
  **************************************************************************/
 
 
-static GTree *canvas_nodes;		/* We don't use the nodes tree directly in order to 
+static GTree *canvas_nodes;	/* We don't use the nodes tree directly in order to 
 				 * separate data from presentation: that is, we need to
 				 * keep a list of CanvasItems, but we do not want to keep
 				 * that info on the nodes tree itself */
-static GTree *canvas_links;		/* See canvas_nodes */
+static GTree *canvas_links;	/* See canvas_nodes */
 
 static guint known_protocols = 0;
 
@@ -100,6 +100,12 @@ static guint known_protocols = 0;
 static gboolean is_idle = FALSE;
 static guint displayed_nodes;
 static GdkColor black_color;
+static gboolean need_reposition = TRUE;	/* Force a diagram relayout */
+static gboolean need_font_refresh = TRUE;/* Force font refresh during layout */
+static gint diagram_timeout;	/* Descriptor of the diagram timeout function
+				 * (Used to change the refresh_period in the callback */
+
+static long canvas_obj_count = 0; /* counter of canvas objects */
 
 /***************************************************************************
  *
@@ -133,10 +139,57 @@ static gint node_item_event (GnomeCanvasItem * item,
 static void update_legend(void);
 
 
+void ask_reposition(gboolean r_font)
+{
+  need_reposition = TRUE;
+  need_font_refresh = r_font;
+}
+
+void dump_stats(guint32 diff_msecs)
+{
+  gchar *status_string;
+  long ipc=ipcache_active_entries();
+  status_string = g_strdup_printf (
+    _("Nodes: %d (on canvas:%d, shown: %u), Links: %d, Conversations: %ld, "
+      "names %ld, protocols %ld. Total Packets seen: %lu (in memory: %d, "
+      "on list %ld). IP cache entries %ld. Canvas objs: %ld. Refreshed: %u ms"),
+                                   node_count(), 
+                                   g_tree_nnodes(canvas_nodes), displayed_nodes, 
+                                   links_catalog_size(), active_conversations(), 
+                                   active_names(), protocol_summary_size(),
+                                   n_packets, total_mem_packets, 
+                                   packet_list_item_count(), ipc,
+                                   canvas_obj_count,
+                                   (unsigned int) diff_msecs);
+  
+  g_my_info (status_string);
+  g_free(status_string);
+}
+
+/* called when a watched object is finalized */
+static void finalize_callback(gpointer data, GObject *obj)
+{
+  --canvas_obj_count;
+}
+/* increase reference to object and optionally register a callback to check 
+ * for reference leaks */
+static void addref_canvas_obj(GObject *obj)
+{
+  g_assert(obj);
+  g_object_ref_sink(obj);
+
+  if (INFO_ENABLED)
+    {
+      /* to check for resource leaks, we ask for a notify ... */
+      g_object_weak_ref(obj, finalize_callback, NULL);
+      ++canvas_obj_count;
+    }
+}
+
 /* It updates controls from values of variables, and connects control
  * signals to callback functions */
 void
-init_diagram ()
+init_diagram (GladeXML *xml)
 {
   GtkStyle *style;
   GtkWidget *canvas;
@@ -148,7 +201,6 @@ init_diagram ()
                             NULL, NULL, (GDestroyNotify)canvas_link_delete);
 
   initialize_pref_controls();
-
   
   /* Sets canvas background to black */
   canvas = glade_xml_get_widget (xml, "canvas1");
@@ -168,13 +220,24 @@ init_diagram ()
 
   /* Set the already_updating global flag */
   already_updating = FALSE;
+  stop_requested = FALSE;
 }				/* init_diagram */
 
 
+void change_refresh_period(guint32 newperiod)
+{
+  pref.refresh_period = newperiod;
+  /* When removing the source (which could either be an idle or a timeout
+   * function, I'm also forcing the callback for the corresponding 
+   * destroying function, which in turn will install a timeout or idle
+   * function using the new refresh_period. It might take a while for it
+   * to settle down, but I think it works now */
+  g_source_remove (diagram_timeout);
+}
+
 void
 destroying_timeout (gpointer data)
 {
-/* g_message ("A timeout function has been destroyed"); */
   diagram_timeout = g_idle_add_full (G_PRIORITY_DEFAULT,
 				     (GtkFunction) update_diagram,
 				     data, (GDestroyNotify) destroying_idle);
@@ -184,7 +247,6 @@ destroying_timeout (gpointer data)
 void
 destroying_idle (gpointer data)
 {
-/*   g_message ("An idle function has been destroyed"); */
   diagram_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
 					pref.refresh_period,
 					(GtkFunction) update_diagram,
@@ -197,12 +259,6 @@ destroying_idle (gpointer data)
 static void 
 canvas_node_delete(canvas_node_t *canvas_node)
 {
-  if (canvas_node->group_item)
-    {
-      gtk_object_destroy (GTK_OBJECT (canvas_node->group_item));
-      g_object_unref (G_OBJECT (canvas_node->group_item));
-      canvas_node->group_item = NULL;
-    }
   if (canvas_node->node_item)
     {
       gtk_object_destroy (GTK_OBJECT (canvas_node->node_item));
@@ -215,6 +271,12 @@ canvas_node_delete(canvas_node_t *canvas_node)
       g_object_unref (G_OBJECT (canvas_node->text_item));
       canvas_node->text_item = NULL;
     }
+  if (canvas_node->group_item)
+    {
+      gtk_object_destroy (GTK_OBJECT (canvas_node->group_item));
+      g_object_unref (G_OBJECT (canvas_node->group_item));
+      canvas_node->group_item = NULL;
+    }
 
   g_free (canvas_node);
 }
@@ -261,8 +323,6 @@ diagram_update_nodes(GtkWidget * canvas)
   /* free the list - list items are already destroyed */
   g_list_free(delete_list);
 
-  g_my_debug ("Number of nodes %d", g_tree_nnodes (canvas_nodes));
-
   /* Limit the number of nodes displayed, if a limit has been set */
   /* TODO check whether this is the right function to use, now that we have a more
    * general display_node called in update_canvas_nodes */
@@ -275,6 +335,7 @@ diagram_update_nodes(GtkWidget * canvas)
 		       (GTraverseFunc) reposition_canvas_nodes,
 		       canvas);
       need_reposition = FALSE;
+      need_font_refresh = FALSE;
     }
 }
 
@@ -301,8 +362,6 @@ diagram_update_links(GtkWidget * canvas)
 
   /* free the list - list items are already destroyed */
   g_list_free(delete_list);
-
-  g_my_debug ("Number of links %d", g_tree_nnodes (canvas_links));
 }
 
 /* Refreshes the diagram. Called each refresh_period ms
@@ -315,16 +374,18 @@ update_diagram (GtkWidget * canvas)
 {
   static struct timeval last_time = { 0, 0 }, diff;
   guint32 diff_msecs;
+  enum status_t status;
 
+  status = get_capture_status();
   if (status == PAUSE)
     return FALSE;
 
-  if (end_of_file && status != STOP)
+  if (status == CAP_EOF)
     {
-      g_my_debug ("End of file and status != STOP. Pausing.");
-      gui_pause_capture ();
+      gui_eof_capture ();
+      return FALSE;
     }
-
+  
   /* 
    * It could happen that during an intensive calculation, in order
    * to update the GUI and make the application responsive gtk_main_iteration
@@ -354,6 +415,9 @@ update_diagram (GtkWidget * canvas)
   /* update proto legend */
   update_legend();
 
+  /* Now update info windows */
+  update_info_windows ();
+
   /* With this we make sure that we don't overload the
    * CPU with redraws */
 
@@ -369,44 +433,22 @@ update_diagram (GtkWidget * canvas)
   diff_msecs = diff.tv_sec * 1000 + diff.tv_usec / 1000;
   last_time = now;
 
-  if (pref.is_debug)
-    {
-      GString *status_string= g_string_new (_("Number of nodes: "));
-      g_string_sprintfa (status_string, "%d", g_tree_nnodes(canvas_nodes));
-    
-      g_string_sprintfa (status_string,
-                         _(". Refresh Period: %d"), (int) diff_msecs);
-    
-      g_string_sprintfa (status_string,
-                         ". Total Packets %g, packets in memory: %g", n_packets,
-                         total_mem_packets);
-      if (is_idle)
-        status_string = g_string_append (status_string, _(". IDLE."));
-      else
-        status_string = g_string_append (status_string, _(". TIMEOUT."));
-      g_my_debug (status_string->str);
-      g_string_free (status_string, TRUE);
-    }
-
   already_updating = FALSE;
 
   if (!is_idle)
     {
       if (diff_msecs > pref.refresh_period * 1.2)
-	{
-/* 	  g_message ("Timeout about to be removed"); */
 	  return FALSE;		/* Removes the timeout */
-	}
     }
   else
     {
       if (diff_msecs < pref.refresh_period)
-	{
-/*	  g_message ("Idle about to be removed"); */
 	  return FALSE;		/* removes the idle */
-	}
     }
 
+  if (stop_requested)
+    gui_stop_capture();
+  
   return TRUE;			/* Keep on calling this function */
 }				/* update_diagram */
 
@@ -449,18 +491,20 @@ check_new_protocol (GtkWidget *prot_table, const protostack_t *pstk)
 {
   const GList *protocol_item;
   const protocol_t *protocol;
-  GtkStyle *style;
   GdkColor color;
+  GtkStyle *style;
   GtkLabel *lab;
   GtkWidget *newlab;
-  const GList *childlist;
+  GList *childlist;
 
   if (!pstk)
     return; /* nothing to do */
 
+  childlist = gtk_container_get_children(GTK_CONTAINER(prot_table));
   protocol_item = pstk->protostack[pref.stack_level];
   while (protocol_item)
     {
+      const GList *cur;
       protocol = protocol_item->data;
 
       /* prepare next */
@@ -468,23 +512,20 @@ check_new_protocol (GtkWidget *prot_table, const protostack_t *pstk)
 
       /* First, we check whether the diagram already knows about this protocol,
        * checking whether it is shown on the legend. */
-      childlist = gtk_container_get_children(GTK_CONTAINER(prot_table));
-      while (childlist)
+      cur = childlist;
+      while (cur)
         {
-          lab = GTK_LABEL(childlist->data);
+          lab = GTK_LABEL(cur->data);
           if (lab && !strcmp(protocol->name, gtk_label_get_label(lab)))
             break; /* found */
-          childlist = childlist->next;
+          cur = cur->next;
         }
       
-      if (childlist)
-        {
-          g_my_debug ("Protocol %s found in legend protocols list",
-                      protocol->name);
+      if (cur)
           continue; /* found, skip to next */
-        }
 
-      g_my_debug ("Protocol not found. Creating legend item");
+      g_my_debug ("Protocol '%s' not found. Creating legend item", 
+                  protocol->name);
     
       /* It's not, so we build a new entry on the legend */
     
@@ -493,9 +534,9 @@ check_new_protocol (GtkWidget *prot_table, const protostack_t *pstk)
       gtk_widget_show (newlab);
       gtk_misc_set_alignment(GTK_MISC(newlab), 0, 0);
 
-      color = protohash_get(protocol->name);
+      color = protohash_color(protocol->name);
       if (!gdk_colormap_alloc_color
-          (gdk_colormap_get_system (), &color, FALSE, TRUE))
+          (gdk_colormap_get_system(), &color, FALSE, TRUE))
         g_warning (_("Unable to allocate color for new protocol %s"),
                    protocol->name);
 
@@ -507,6 +548,7 @@ check_new_protocol (GtkWidget *prot_table, const protostack_t *pstk)
       gtk_container_add(GTK_CONTAINER(prot_table), newlab);
       known_protocols++;
     }
+  g_list_free(childlist);
 }				/* check_new_protocol */
 
 /* empties the table of protocols */
@@ -548,16 +590,19 @@ check_new_node (node_t * node, GtkWidget * canvas)
 
   if (display_node (node) && !g_tree_lookup (canvas_nodes, &node->node_id))
     {
-      group = gnome_canvas_root (GNOME_CANVAS (canvas));
-
       new_canvas_node = g_malloc (sizeof (canvas_node_t));
+      g_assert(new_canvas_node);
+      
       new_canvas_node->canvas_node_id = node->node_id;
 
+      /* Create a new group to hold the node and its labels */
+      group = gnome_canvas_root (GNOME_CANVAS (canvas));
       group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (group,
-							 gnome_canvas_group_get_type
-							 (), "x", 100.0, "y",
-							 100.0, NULL));
-      g_object_ref (G_OBJECT (group));
+							 GNOME_TYPE_CANVAS_GROUP,
+							 "x", 100.0, 
+                                                         "y", 100.0, NULL));
+      addref_canvas_obj(G_OBJECT (group));
+      new_canvas_node->group_item = group;
 
       new_canvas_node->node_item
 	= gnome_canvas_item_new (group,
@@ -566,10 +611,11 @@ check_new_node (node_t * node, GtkWidget * canvas)
 				 "x2", 0.0,
 				 "y1", 0.0,
 				 "y2", 0.0,
-				 "fill_color", pref.node_color,
+				 "fill_color", "white",
 				 "outline_color", "black",
 				 "width_pixels", 0, NULL);
-      g_object_ref (G_OBJECT (new_canvas_node->node_item));
+      addref_canvas_obj(G_OBJECT (new_canvas_node->node_item));
+
       new_canvas_node->text_item =
 	gnome_canvas_item_new (group, GNOME_TYPE_CANVAS_TEXT,
 			       "text", node->name->str,
@@ -578,20 +624,16 @@ check_new_node (node_t * node, GtkWidget * canvas)
 			       "anchor", GTK_ANCHOR_CENTER,
 			       "font", pref.fontname,
 			       "fill_color", pref.text_color, NULL);
-      g_object_ref (G_OBJECT (new_canvas_node->text_item));
-      new_canvas_node->group_item = group;
+      addref_canvas_obj(G_OBJECT (new_canvas_node->text_item));
 
       gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM
 				      (new_canvas_node->text_item));
       g_signal_connect (G_OBJECT (new_canvas_node->group_item), "event",
 			(GtkSignalFunc) node_item_event, new_canvas_node);
 
-      g_tree_insert (canvas_nodes,
-		     &new_canvas_node->canvas_node_id, new_canvas_node);
-      g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-	     _("Creating canvas_node: %s. Number of nodes %d"),
-	     node->name->str, g_tree_nnodes (canvas_nodes));
-
+      if (!new_canvas_node->node_item || !new_canvas_node->text_item)
+        g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, _("Canvas node null"));
+      
       /*
        * We hide them until we are sure that they will get a proper position
        * in reposition_nodes
@@ -599,9 +641,15 @@ check_new_node (node_t * node, GtkWidget * canvas)
       gnome_canvas_item_hide (new_canvas_node->node_item);
       gnome_canvas_item_hide (new_canvas_node->text_item);
 
-
       new_canvas_node->is_new = TRUE;
       new_canvas_node->shown = TRUE;
+
+      g_tree_insert (canvas_nodes,
+		     &new_canvas_node->canvas_node_id, new_canvas_node);
+      g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+	     _("Creating canvas_node: %s. Number of nodes %d"),
+	     node->name->str, g_tree_nnodes (canvas_nodes));
+
       need_reposition = TRUE;
     }
 
@@ -664,7 +712,7 @@ canvas_node_update(node_id_t * node_id, canvas_node_t * canvas_node,
 
   if (node->main_prot[pref.stack_level])
     {
-      canvas_node->color = protohash_get(node->main_prot[pref.stack_level]);
+      canvas_node->color = protohash_color(node->main_prot[pref.stack_level]);
 
       gnome_canvas_item_set (canvas_node->node_item,
 			     "x1", -node_size / 2,
@@ -689,16 +737,16 @@ canvas_node_update(node_id_t * node_id, canvas_node_t * canvas_node,
   /*TODO why is it exactly that sometimes it is NULL? */
   if (canvas_node->text_item)
     {
-      g_object_get (G_OBJECT (canvas_node->text_item), "text", &nametmp,
+      g_object_get (G_OBJECT (canvas_node->text_item), 
+                    "text", &nametmp,
 		    NULL);
       if (strcmp (nametmp, node->name->str))
 	{
 	  gnome_canvas_item_set (canvas_node->text_item,
-				 "text", node->name->str, NULL);
+				 "text", node->name->str, 
+                                 NULL);
 	  gnome_canvas_item_request_update (canvas_node->text_item);
 	}
-
-      /* Memprof is telling us that we have to free the string */
       g_free (nametmp);
     }
 
@@ -854,10 +902,7 @@ reposition_canvas_nodes (guint8 * ether_addr, canvas_node_t * canvas_node,
   gnome_canvas_get_scroll_region (GNOME_CANVAS (canvas),
 				  &xmin, &ymin, &xmax, &ymax);
   if (!n_nodes)
-    {
       n_nodes = node_i = displayed_nodes;
-      g_my_debug ("Displayed nodes = %d", displayed_nodes);
-    }
 
   xmin += text_compensation;
   xmax -= text_compensation;	/* Reduce the drawable area so that
@@ -916,15 +961,20 @@ reposition_canvas_nodes (guint8 * ether_addr, canvas_node_t * canvas_node,
 
   if (!pref.stationary || canvas_node->is_new)
     {
-      /* g_message ("%g %g", x, y); */
       gnome_canvas_item_set (GNOME_CANVAS_ITEM (canvas_node->group_item),
 			     "x", x, "y", y, NULL);
       canvas_node->is_new = FALSE;
     }
 
-  /* We update the text font */
-  gnome_canvas_item_set (canvas_node->text_item, "font", pref.fontname, NULL);
-
+  if (need_font_refresh)
+    {
+      /* We update the text font */
+      gnome_canvas_item_set (canvas_node->text_item, 
+                             "font", pref.fontname, 
+                             "fill_color", pref.text_color, 
+                             NULL);
+    }
+  
   if (pref.diagram_only)
     {
       gnome_canvas_item_hide (canvas_node->text_item);
@@ -938,7 +988,6 @@ reposition_canvas_nodes (guint8 * ether_addr, canvas_node_t * canvas_node,
   gnome_canvas_item_show (canvas_node->node_item);
   gnome_canvas_item_request_update (canvas_node->node_item);
 
-
   node_i--;
 
   if (node_i)
@@ -968,6 +1017,7 @@ check_new_link (link_id_t * link_id, link_t * link, GtkWidget * canvas)
       group = gnome_canvas_root (GNOME_CANVAS (canvas));
 
       new_canvas_link = g_malloc (sizeof (canvas_link_t));
+      g_assert(new_canvas_link);
       new_canvas_link->canvas_link_id = *link_id;
 
       /* We set the lines position using groups positions */
@@ -980,7 +1030,7 @@ check_new_link (link_id_t * link_id, link_t * link, GtkWidget * canvas)
 	= gnome_canvas_item_new (group,
 				 gnome_canvas_polygon_get_type (),
 				 "points", points, "fill_color", "tan", NULL);
-      g_object_ref (G_OBJECT (new_canvas_link->link_item));
+      addref_canvas_obj(G_OBJECT (new_canvas_link->link_item));
 
 
       g_tree_insert (canvas_links,
@@ -992,16 +1042,6 @@ check_new_link (link_id_t * link_id, link_t * link, GtkWidget * canvas)
       g_signal_connect (G_OBJECT (new_canvas_link->link_item), "event",
 			(GtkSignalFunc) link_item_event, new_canvas_link);
 
-
-      if (pref.is_debug)
-      {
-        gchar *str = link_id_node_names(link_id);
-        g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-               _("Creating canvas_link: %s. Number of links %d"),
-               str, g_tree_nnodes (canvas_links));
-        g_free(str);
-      }
-
     }
 
   return FALSE;
@@ -1081,20 +1121,15 @@ canvas_link_update(link_id_t * link_id, canvas_link_t * canvas_link,
    * I have to initialize canvas_link->color to a known value */
   if (link->main_prot[pref.stack_level])
     {
-      canvas_link->color = protohash_get(link->main_prot[pref.stack_level]);
+      canvas_link->color = protohash_color(link->main_prot[pref.stack_level]);
 
-      if (pref.nofade)
-        scale = 1.0;
-      else
-	{
-	  /* scale color down to 10% at link timeout */
-          struct timeval diff;
-          diff = substract_times (now, link->link_stats.stats.last_time);
-	  scale =
-	    pow (0.10,
-		 (diff.tv_sec * 1000.0 +
-		  diff.tv_usec / 1000) / pref.gui_link_timeout_time);
-	}
+      /* scale color down to 10% at link timeout */
+      struct timeval diff;
+      diff = substract_times (now, link->link_stats.stats.last_time);
+      scale =
+        pow (0.10,
+             (diff.tv_sec * 1000.0 +
+              diff.tv_usec / 1000) / pref.gui_link_timeout_time);
 
       scaledColor =
         (((int) (scale * canvas_link->color.red) & 0xFF00) << 16) |
@@ -1138,7 +1173,7 @@ get_node_size (gdouble average)
       result = sqrt (average + 1);
       break;
     }
-  return (double) (5 + pref.node_radius_multiplier * result);
+  return 5.0 + pref.node_radius_multiplier * result;
 }
 
 /* Returs the width in pixels given average traffic and size mode */
@@ -1158,24 +1193,20 @@ get_link_size (gdouble average)
       result = sqrt (average + 1);
       break;
     }
-  return (double) (1 + pref.link_width_multiplier * result);
+  return 1.0 + pref.node_radius_multiplier * pref.link_node_ratio * result;
 }				/* get_link_size */
 
 
 
 /* Called for every event a link receives. Right now it's used to 
- * set a message in the appbar */
+ * set a message in the statusbar */
 static gint
 link_item_event (GnomeCanvasItem * item, GdkEvent * event,
 		 canvas_link_t * canvas_link)
 {
-  static GnomeAppBar *appbar;
   gchar *str;
   const link_t *link=NULL;
 
-  if (!appbar)
-    appbar = GNOME_APPBAR (glade_xml_get_widget (xml, "appbar1"));
-
   switch (event->type)
     {
     case GDK_2BUTTON_PRESS:
@@ -1191,11 +1222,11 @@ link_item_event (GnomeCanvasItem * item, GdkEvent * event,
 			   link->main_prot[pref.stack_level]);
       else
 	str = g_strdup_printf (_("Link main protocol unknown"));
-      gnome_appbar_push (appbar, str);
+      gtk_statusbar_push(statusbar, 1, str);
       g_free (str);
       break;
     case GDK_LEAVE_NOTIFY:
-      gnome_appbar_pop (appbar);
+      gtk_statusbar_pop(statusbar, 1);
       break;
     default:
       break;
@@ -1206,7 +1237,7 @@ link_item_event (GnomeCanvasItem * item, GdkEvent * event,
 
 
 /* Called for every event a node receives. Right now it's used to 
- * set a message in the appbar and launch the popup timeout */
+ * set a message in the statusbar and launch the popup timeout */
 static gint
 node_item_event (GnomeCanvasItem * item, GdkEvent * event,
 		 canvas_node_t * canvas_node)
@@ -1214,10 +1245,6 @@ node_item_event (GnomeCanvasItem * item, GdkEvent * event,
 
   gdouble item_x, item_y;
   const node_t *node = NULL;
-  static GnomeAppBar *appbar;
-
-  if (!appbar)
-    appbar = GNOME_APPBAR (glade_xml_get_widget (xml, "appbar1"));
 
   /* This is not used yet, but it will be. */
   item_x = event->button.x;
@@ -1232,13 +1259,15 @@ node_item_event (GnomeCanvasItem * item, GdkEvent * event,
         node = nodes_catalog_find(&canvas_node->canvas_node_id);
       if (node)
         {
-          if (pref.new_infodlg)
-            node_protocols_window_create( &canvas_node->canvas_node_id );
-          else
-            node_info_window_create( &canvas_node->canvas_node_id );
-          g_my_info ("Nodes: %d. Canvas nodes: %d", nodes_catalog_size(),
-                     nodes_catalog_size());
-          node_dump(node);
+          node_protocols_window_create( &canvas_node->canvas_node_id );
+          g_my_info ("Nodes: %d (shown %u)", nodes_catalog_size(),
+                     displayed_nodes);
+          if (DEBUG_ENABLED)
+            {
+              gchar *msg = node_dump(node);
+              g_my_debug(msg);
+              g_free(msg);
+            }
         }
       break;
     default:
@@ -1249,12 +1278,10 @@ node_item_event (GnomeCanvasItem * item, GdkEvent * event,
 
 }				/* node_item_event */
 
-/* Pushes a string into the appbar status area */
-
+/* Pushes a string into the statusbar stack */
 void
-set_appbar_status (gchar * str)
+set_statusbar_msg (gchar * str)
 {
-  static GnomeAppBar *appbar = NULL;
   static gchar *status_string = NULL;
 
   if (status_string)
@@ -1262,47 +1289,10 @@ set_appbar_status (gchar * str)
 
   status_string = g_strdup (str);
 
-  if (!appbar)
-    appbar = GNOME_APPBAR (glade_xml_get_widget (xml, "appbar1"));
+  gtk_statusbar_pop(statusbar, 0);
+  gtk_statusbar_push(statusbar, 0, status_string);
+}				/* set_statusbar_msg */
 
-  gnome_appbar_pop (appbar);
-  gnome_appbar_push (appbar, status_string);
-
-}				/* set_appbar_status */
-
-gchar *
-traffic_to_str (gdouble traffic, gboolean is_speed)
-{
-  static gchar *str = NULL;
-
-  if (str)
-    g_free (str);
-
-  if (is_speed)
-    {
-      if (traffic > 1000000)
-	str = g_strdup_printf ("%.3f Mbps", traffic / 1000000);
-      else if (traffic > 1000)
-	str = g_strdup_printf ("%.3f Kbps", traffic / 1000);
-      else
-	str = g_strdup_printf ("%.0f bps", traffic);
-    }
-  else
-    {
-      /* Debug code for sanity check */
-      if (traffic && traffic < 1)
-	g_warning ("Ill traffic value in traffic_to_str");
-
-      if (traffic > 1024 * 1024)
-	str = g_strdup_printf ("%.3f Mbytes", traffic / 1024 / 1024);
-      else if (traffic > 1024)
-	str = g_strdup_printf ("%.3f Kbytes", traffic / 1024);
-      else
-	str = g_strdup_printf ("%.0f bytes", traffic);
-    }
-
-  return str;
-}				/* traffic_to_str */
 
 static gint canvas_node_compare(const node_id_t *a, const node_id_t *b,
                                 gpointer dummy)
