@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: diagram.h,v 1.27 2006/01/01 17:05:26 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,14 @@
 
 
 gboolean already_updating;	/* True while an instance of update_diagram is running */
+gboolean stop_requested;	/* True if there is a pending stop request */
 
 guint update_diagram (GtkWidget * canvas);
-void init_diagram (void);
+void init_diagram (GladeXML *xml);
 void destroying_timeout (gpointer data);
 void destroying_idle (gpointer data);
-void set_appbar_status (gchar * str);
+void set_statusbar_msg (gchar * str);
 void delete_gui_protocols (void);
-gchar *traffic_to_str (gdouble traffic, gboolean is_speed);
+void ask_reposition(gboolean refresh_font); /* request diagram relayout */
+void change_refresh_period(guint32 newperiod); 
+void dump_stats(guint32 diff_msecs);
