@@ -1,7 +1,7 @@
 /* eth_resolv.h
  * Definitions for network object lookup
  *
- * $Id: eth_resolv.h,v 1.4 2005/12/30 16:36:09 bchiara Exp $
+ * $Id$
  *
  * Originally by Laurent Deniel <deniel@worldnet.fr>
  * Modified for etherape by Juan Toledo <toledo@users.sourceforge.net>
@@ -31,9 +31,6 @@
 
 /* Functions in resolv.c */
 
-/* get_udp_port returns the UDP port name or "%d" if not found */
-extern char *get_udp_port (u_int port);
-
 /* get_tcp_port returns the TCP port name or "%d" if not found */
 extern char *get_tcp_port (u_int port);
 
@@ -42,7 +39,4 @@ extern char *get_tcp_port (u_int port);
    "%02x:%02x:%02x:%02x:%02x:%02x" */
 extern char *get_ether_name (const u_char * addr);
 
-/* get_manuf_name returns the vendor name or "%02x:%02x:%02x" if not known */
-extern char *get_manuf_name (u_char * addr);
-
 #endif /* __ETH_RESOLV_H__ */
