@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: info_windows.c,v 1.24 2006/01/12 21:13:42 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,22 @@
 #include "node.h"
 #include "datastructs.h"
 #include "protocols.h"
+#include "capture.h"
+#include "prot_types.h"
+#include "ui_utils.h"
+#include "node_windows.h"
+
+typedef enum 
+{
+  PROTO_COLUMN_COLOR = 0,
+  PROTO_COLUMN_NAME,
+  PROTO_COLUMN_PORT,
+  PROTO_COLUMN_INSTANTANEOUS,
+  PROTO_COLUMN_ACCUMULATED,
+  PROTO_COLUMN_LASTHEARD,
+  PROTO_COLUMN_PACKETS,
+  PROTO_COLUMN_N        /* must be the last entry */
+} PROTO_COLUMN;
 
 typedef struct
 {
@@ -40,6 +56,7 @@ prot_info_window_t;
 typedef struct protocol_list_item_t_tag
 {
   gchar *name; /* protocol name */
+  uint port;     /* protocol port */
   GdkColor color; /* protocol color */
   basic_stats_t rowstats; 
 } protocol_list_item_t;
@@ -53,7 +70,6 @@ static GList *prot_info_windows = NULL;
 
 /* private functions */
 static void update_prot_info_windows (void);
-static void update_node_info_window (GtkWidget *node_window);
 static void update_prot_info_window (prot_info_window_t * prot_info_window);
 static gint node_info_window_compare (gconstpointer a, gconstpointer b);
 static gint prot_info_compare (gconstpointer a, gconstpointer b);
@@ -61,77 +77,10 @@ static void update_stats_info_windows (void);
 static void update_node_protocols_window (GtkWidget *node_window);
 static void update_link_info_window (GtkWidget *window);
 
-/* 
-  
-  Helper functions 
-
-*/
-
-static gchar *
-timeval_to_str (struct timeval last_heard)
-{
-  static gchar *str = NULL;
-  struct timeval diff;
-  struct tm broken_time;
-
-  if (str)
-    g_free (str);
-
-  diff = substract_times (now, last_heard);
-  if (!localtime_r ((time_t *) & (last_heard.tv_sec), &broken_time))
-    {
-      g_my_critical ("Time conversion failed in timeval_to_str");
-      return NULL;
-    }
-
-  if (diff.tv_sec <= 60)
-    {
-      /* Meaning "n seconds" ago */
-      str = g_strdup_printf (_("%d\" ago"), (int) diff.tv_sec);
-    }
-  else if (diff.tv_sec < 600)
-    {
-      /* Meaning "m minutes, n seconds ago" */
-      str =
-	g_strdup_printf (_("%d'%d\" ago"),
-			 (int) floor ((double) diff.tv_sec / 60),
-			 (int) diff.tv_sec % 60);
-    }
-  else if (diff.tv_sec < 3600 * 24)
-    {
-      str =
-	g_strdup_printf ("%d:%d", broken_time.tm_hour, broken_time.tm_min);
-    }
-  else
-    {
-      /* Watch out! The first is month, the second day of the month */
-      str = g_strdup_printf (_("%d/%d %d:%d"),
-			     broken_time.tm_mon, broken_time.tm_mday,
-			     broken_time.tm_hour, broken_time.tm_min);
-    }
-
-  return str;
-}				/* timeval_to_str */
-
-/* registers the named glade widget on the specified object */
-static void
-register_glade_widget(GladeXML *xm, GObject *tgt, const gchar *widgetName)
-{
-  GtkWidget *widget;
-  widget = glade_xml_get_widget (xm, widgetName);
-  g_object_set_data (tgt, widgetName, widget);
-}
-
-static void
-update_gtklabel(GtkWidget *window, const gchar *lblname, const gchar *value)
-{
-  GtkWidget *widget = g_object_get_data (G_OBJECT (window), lblname);
-  gtk_label_set_text (GTK_LABEL (widget), value);
-}
-
 /* ----------------------------------------------------------
 
-   Protocol Info Detail window functions (prot_info_windows)
+   Generic Protocol Info Detail window functions (prot_info_windows)
+   Used to display nodes and links detail data (and protos)
 
    ---------------------------------------------------------- */
 /* Comparison function used to compare prot_info_windows */
@@ -158,11 +107,11 @@ create_prot_info_window (protocol_t * protocol)
 			    protocol->name, prot_info_compare)))
     {
       xml_info_window =
-	glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "prot_info", NULL);
+	glade_xml_new (pref.glade_file, "prot_info", NULL);
       if (!xml_info_window)
 	{
 	  g_error (_("We could not load the interface! (%s)"),
-		   GLADEDIR "/" ETHERAPE_GLADE_FILE);
+		   pref.glade_file);
 	  return;
 	}
       glade_xml_signal_autoconnect (xml_info_window);
@@ -237,6 +186,7 @@ update_prot_info_window (prot_info_window_t * prot_info_window)
   const protocol_t *prot;
   GtkWidget *window;
   GtkWidget *widget;
+  gchar *str;
 
   window = prot_info_window->window;
   if (!window)
@@ -258,13 +208,17 @@ update_prot_info_window (prot_info_window_t * prot_info_window)
   widget = g_object_get_data (G_OBJECT (window), "name_label");
   gtk_label_set_text (GTK_LABEL (widget), prot_info_window->prot_name);
   widget = g_object_get_data (G_OBJECT (window), "last_heard_label");
-  gtk_label_set_text (GTK_LABEL (widget), timeval_to_str (prot->last_heard));
+  str = timeval_to_str (prot->stats.last_time);
+  gtk_label_set_text (GTK_LABEL (widget), str);
+  g_free(str);
   widget = g_object_get_data (G_OBJECT (window), "average");
-  gtk_label_set_text (GTK_LABEL (widget),
-		      traffic_to_str (prot->average, TRUE));
+  str = traffic_to_str (prot->stats.average, TRUE);
+  gtk_label_set_text (GTK_LABEL (widget), str);
+  g_free(str);
   widget = g_object_get_data (G_OBJECT (window), "accumulated");
-  gtk_label_set_text (GTK_LABEL (widget),
-		      traffic_to_str (prot->accumulated, FALSE));
+  str = traffic_to_str (prot->stats.accumulated, FALSE);
+  gtk_label_set_text (GTK_LABEL (widget), str);
+  g_free(str);
   gtk_widget_queue_resize (GTK_WIDGET (prot_info_window->window));
 
 }				/* update_prot_info_window */
@@ -275,6 +229,9 @@ update_prot_info_windows (void)
 {
   GList *list_item = NULL, *remove_item;
   prot_info_window_t *prot_info_window = NULL;
+  enum status_t status;
+
+  status = get_capture_status();
 
   list_item = prot_info_windows;
   while (list_item)
@@ -296,7 +253,6 @@ update_prot_info_windows (void)
 	}
     }
 
-  return;
 }				/* update_prot_info_windows */
 
 
@@ -325,8 +281,8 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
 
   /* reads the proto ptr from 6th columns */
   const protocol_list_item_t *prot1, *prot2;
-  gtk_tree_model_get (gs, a, 5, &prot1, -1);
-  gtk_tree_model_get (gs, b, 5, &prot2, -1);
+  gtk_tree_model_get (gs, a, PROTO_COLUMN_N, &prot1, -1);
+  gtk_tree_model_get (gs, b, PROTO_COLUMN_N, &prot2, -1);
   if (!prot1 || !prot2)
     return 0;			/* error */
 
@@ -336,12 +292,13 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
 
   switch (idcol)
     {
-    case 0: /* name */
-    case 6: /* color */
+    case PROTO_COLUMN_COLOR: /* color */
+    case PROTO_COLUMN_PORT: /* port */
+    case PROTO_COLUMN_NAME: /* name */
     default:
       ret = strcmp (prot1->name, prot2->name);
       break;
-    case 1:
+    case PROTO_COLUMN_INSTANTANEOUS:
       t1 = prot1->rowstats.average;
       t2 = prot2->rowstats.average;
       if (t1 == t2)
@@ -351,7 +308,7 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
       else
 	ret = 1;
       break;
-    case 2:
+    case PROTO_COLUMN_ACCUMULATED:
       t1 = prot1->rowstats.accumulated;
       t2 = prot2->rowstats.accumulated;
       if (t1 == t2)
@@ -361,7 +318,7 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
       else
 	ret = 1;
       break;
-    case 3:
+    case PROTO_COLUMN_LASTHEARD:
       time1 = prot1->rowstats.last_time;
       time2 = prot2->rowstats.last_time;
       diff = substract_times (time1, time2);
@@ -372,7 +329,7 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
       else
 	ret = 1;
       break;
-    case 4:
+    case PROTO_COLUMN_PACKETS:
       if (prot1->rowstats.accu_packets == prot2->rowstats.accu_packets)
 	ret = 0;
       else if (prot1->rowstats.accu_packets < prot2->rowstats.accu_packets)
@@ -385,88 +342,131 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
   return ret;
 }
 
-static void
-create_protocols_table (GtkWidget *window, GtkTreeView *gv)
+static GtkListStore *create_protocols_table (GtkWidget *window)
 {
+  GtkTreeView *gv;
   GtkListStore *gs;
   GtkTreeViewColumn *gc;
+  int i;
 
-  /* create the store  - it uses 7 values, five displayable, one ptr, and the
-     proto color */
-  gs = gtk_list_store_new (7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-			   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER,
-			   GDK_TYPE_COLOR);
+  /* get the treeview, if present */
+  gv = retrieve_treeview(window);
+  if (!gv)
+    return NULL; 
+
+  gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
+  if (gs)
+    return gs;
+
+  /* create the store  - it uses 8 values, six displayable, one proto color 
+     and the data pointer */
+  gs = gtk_list_store_new (8, GDK_TYPE_COLOR, G_TYPE_STRING, G_TYPE_STRING,
+			   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+			   G_TYPE_STRING, G_TYPE_POINTER);
   gtk_tree_view_set_model (gv, GTK_TREE_MODEL (gs));
 
   /* the view columns and cell renderers must be also created ... 
-     the first column is the proto color  */
+     the first column is the proto color; is a text column wich renders
+     only a space with a colored background. We must set some special properties
+   */
+  
   gc = gtk_tree_view_column_new_with_attributes
-    (" ", gtk_cell_renderer_text_new (), "background-gdk", 6, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
+    (" ", gtk_cell_renderer_text_new(), "background-gdk", PROTO_COLUMN_COLOR, 
+     NULL);
+  g_object_set (G_OBJECT (gc), "resizable", TRUE, 
+                               "reorderable", TRUE, 
+                               NULL);
+//  gtk_tree_view_column_set_sort_column_id(gc, PROTO_COLUMN_COLOR);
   gtk_tree_view_append_column (gv, gc);
 
-  gc = gtk_tree_view_column_new_with_attributes
-    ("Protocol", gtk_cell_renderer_text_new (), "text", 0, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
-  gtk_tree_view_column_set_sort_column_id (gc, 0);
-  gtk_tree_view_append_column (gv, gc);
 
-  gc = gtk_tree_view_column_new_with_attributes
-    ("Inst Traffic", gtk_cell_renderer_text_new (), "text", 1, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
-  gtk_tree_view_column_set_sort_column_id (gc, 1);
-  gtk_tree_view_append_column (gv, gc);
-
-  gc = gtk_tree_view_column_new_with_attributes
-    ("Accum Traffic", gtk_cell_renderer_text_new (), "text", 2, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
-  gtk_tree_view_column_set_sort_column_id (gc, 2);
-  gtk_tree_view_append_column (gv, gc);
-
-  gc = gtk_tree_view_column_new_with_attributes
-    ("Last Heard", gtk_cell_renderer_text_new (), "text", 3, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
-  gtk_tree_view_column_set_sort_column_id (gc, 3);
-  gtk_tree_view_append_column (gv, gc);
-
-  gc = gtk_tree_view_column_new_with_attributes
-    ("Packets", gtk_cell_renderer_text_new (), "text", 4, NULL);
-  g_object_set (G_OBJECT (gc), "resizable", TRUE, "reorderable", TRUE, NULL);
-  gtk_tree_view_column_set_sort_column_id (gc, 4);
-  gtk_tree_view_append_column (gv, gc);
+  create_add_text_column(gv, "Protocol", PROTO_COLUMN_NAME, FALSE);
+  create_add_text_column(gv, "Port", PROTO_COLUMN_PORT, TRUE);
+  create_add_text_column(gv, "Inst Traffic", PROTO_COLUMN_INSTANTANEOUS, FALSE);
+  create_add_text_column(gv, "Accum Traffic", PROTO_COLUMN_ACCUMULATED, FALSE);
+  create_add_text_column(gv, "Last Heard", PROTO_COLUMN_LASTHEARD, FALSE);
+  create_add_text_column(gv, "Packets", PROTO_COLUMN_PACKETS, FALSE);
 
   /* the sort functions ... */
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), 0,
-				   protocols_table_compare, gs, NULL);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), 1,
-				   protocols_table_compare, gs, NULL);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), 2,
-				   protocols_table_compare, gs, NULL);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), 3,
-				   protocols_table_compare, gs, NULL);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), 4,
-				   protocols_table_compare, gs, NULL);
+  for (i = 0 ; i < PROTO_COLUMN_N ; ++i)
+    gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (gs), i,
+                                    protocols_table_compare, gs, NULL);
 
   /* initial sort order is by protocol */
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (gs), 0,
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (gs), 
+                                        PROTO_COLUMN_NAME,
 					GTK_SORT_ASCENDING);
 
-  /* register the treeview in the window */
-  g_object_set_data ( G_OBJECT(window), "proto_clist", gv);
+  return gs;
 }
 
-static void
-update_protocols_table(GtkListStore *gs, const protostack_t *pstk)
+static void protocols_table_clear(GtkListStore *gs)
 {
+  gboolean res;
+  GtkTreeIter it;
+
+  res = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (gs), &it);
+  while (res)
+    {
+      protocol_list_item_t *row_proto = NULL;
+      gtk_tree_model_get (GTK_TREE_MODEL (gs), &it, PROTO_COLUMN_N, 
+                          &row_proto, -1);
+      if (row_proto)
+        {
+          g_free(row_proto->name);
+          g_free(row_proto);
+          res = gtk_list_store_remove (gs, &it);
+        }
+    }
+}
+
+static void update_protocols_row(GtkListStore *gs, GtkTreeIter *it, 
+                                 const protocol_list_item_t *row_proto)
+{
+  gchar *ga, *gb, *gc, *gd, *ge;
+  const port_service_t *ps;
+  
+  ga = traffic_to_str (row_proto->rowstats.accumulated, FALSE);
+  gb = g_strdup_printf ("%lu", row_proto->rowstats.accu_packets);
+  ps = services_port_find(row_proto->name);
+  if (ps)
+    gc = g_strdup_printf ("%d", ps->port);
+  else
+    gc = g_strdup("-");
+  gd = traffic_to_str (row_proto->rowstats.average, TRUE);
+  ge = timeval_to_str (row_proto->rowstats.last_time);
+
+  gtk_list_store_set (gs, it, 
+                      PROTO_COLUMN_ACCUMULATED, ga, 
+                      PROTO_COLUMN_PACKETS, gb,
+                      PROTO_COLUMN_PORT, gc,
+                      PROTO_COLUMN_INSTANTANEOUS, gd,
+                      PROTO_COLUMN_LASTHEARD, ge, -1);
+
+  g_free(ga);
+  g_free(gb);
+  g_free(gc);
+  g_free(gd);
+  g_free(ge);
+}
+
+static void update_protocols_table(GtkWidget *window, const protostack_t *pstk)
+{
+  GtkListStore *gs;
   GList *item;
   gchar *str;
   gboolean res;
   GtkTreeIter it;
 
-  if (!gs || !pstk)
+  gs = create_protocols_table (window);
+  if (!gs)
     return; /* nothing to do */
 
-  item = pstk->protostack[pref.stack_level];
+  if (pstk)
+    item = pstk->protostack[pref.stack_level];
+  else
+    item = NULL;
+  
   res = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (gs), &it);
   while (res || item)
     {
@@ -475,14 +475,18 @@ update_protocols_table(GtkListStore *gs, const protostack_t *pstk)
 
       /* retrieve current row proto (if present) */
       if (res)
-        gtk_tree_model_get (GTK_TREE_MODEL (gs), &it, 5, &row_proto, -1);
+        gtk_tree_model_get (GTK_TREE_MODEL (gs), &it, PROTO_COLUMN_N, 
+                            &row_proto, -1);
 
       if ( !item )
         {
-          /* no more protos on stack, current row (remove moves to next) */
-          g_free(row_proto->name);
-          g_free(row_proto);
-          res = gtk_list_store_remove (gs, &it);
+          /* no more protos on stack, del current row (remove moves to next) */
+          if (res)
+            {
+              g_free(row_proto->name);
+              g_free(row_proto);
+              res = gtk_list_store_remove (gs, &it);
+            }
           continue;
         }
 
@@ -493,14 +497,16 @@ update_protocols_table(GtkListStore *gs, const protostack_t *pstk)
         {
           /* current protocol missing on list, create a new row */
 	  row_proto = g_malloc(sizeof(protocol_list_item_t));
+          g_assert(row_proto);
+          
           row_proto->name = g_strdup(stack_proto->name);
-	  row_proto->color = protohash_get(stack_proto->name);
+	  row_proto->color = protohash_color(stack_proto->name);
 
 	  gtk_list_store_append(gs, &it);
 	  gtk_list_store_set (gs, &it, 
-                              0, row_proto->name, 
-                              5, row_proto, 
-                              6, &row_proto->color, 
+                              PROTO_COLUMN_NAME, row_proto->name, 
+                              PROTO_COLUMN_COLOR, &row_proto->color, 
+                              PROTO_COLUMN_N, row_proto, 
                               -1);
         }
       else
@@ -517,23 +523,8 @@ update_protocols_table(GtkListStore *gs, const protostack_t *pstk)
         }
 
       /* everything ok, update stats */
-      row_proto->rowstats.accumulated = stack_proto->accumulated;
-      row_proto->rowstats.accu_packets = stack_proto->proto_packets;
-      row_proto->rowstats.average = stack_proto->average;
-      row_proto->rowstats.last_time = stack_proto->last_heard;
-
-      str = traffic_to_str (row_proto->rowstats.accumulated, FALSE);
-      gtk_list_store_set (gs, &it, 2, str, -1);
-
-      str = g_strdup_printf ("%.0f", row_proto->rowstats.accu_packets);
-      gtk_list_store_set (gs, &it, 4, str, -1);
-      g_free (str);
-
-      str = traffic_to_str (row_proto->rowstats.average, TRUE);
-      gtk_list_store_set (gs, &it, 1, str, -1);
-
-      str = timeval_to_str (row_proto->rowstats.last_time);
-      gtk_list_store_set (gs, &it, 3, str, -1);
+      row_proto->rowstats = stack_proto->stats;
+      update_protocols_row(gs, &it, row_proto);
 
       if (res)
         res = gtk_tree_model_iter_next (GTK_TREE_MODEL (gs), &it);
@@ -548,7 +539,6 @@ activate_protocols_info_column (GtkMenuItem * gm, guint column)
 {
   GtkTreeViewColumn *gc;
   GtkTreeView *gv = GTK_TREE_VIEW (glade_xml_get_widget (xml, "prot_clist"));
-//  GtkTreeView *gv = GTK_TREE_VIEW (g_object_get_data (window, "proto_clist"));
   if (!gv)
     return;			/* no window, no handling */
 
@@ -560,70 +550,78 @@ activate_protocols_info_column (GtkMenuItem * gm, guint column)
 				    (GTK_CHECK_MENU_ITEM (gm)));
 }				/* on_prot_column_view_activate */
 
-
 void
 on_prot_color_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 0);
+  activate_protocols_info_column (gm, PROTO_COLUMN_COLOR);
 }
 
 void
 on_protocol_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 1);
+  activate_protocols_info_column (gm, PROTO_COLUMN_NAME);
+}
+
+void
+on_port_column_activate (GtkMenuItem * gm, gpointer * user_data)
+{
+  activate_protocols_info_column (gm, PROTO_COLUMN_PORT);
 }
 
 void
 on_instant_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 2);
+  activate_protocols_info_column (gm, PROTO_COLUMN_INSTANTANEOUS);
 }
 
 void
 on_accumulated_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 3);
+  activate_protocols_info_column (gm, PROTO_COLUMN_ACCUMULATED);
 }
 
 void
 on_heard_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 4);
+  activate_protocols_info_column (gm, PROTO_COLUMN_LASTHEARD);
 }
 
 void
 on_packets_column_activate (GtkMenuItem * gm, gpointer * user_data)
 {
-  activate_protocols_info_column (gm, 5);
+  activate_protocols_info_column (gm, PROTO_COLUMN_PACKETS);
 }
 
 
 /* ----------------------------------------------------------
 
-   General Protocol Info window functions (protocols_window) 
+   General Protocol Info window functions (protocols_window)
+   Displays the global protocol table
 
    ---------------------------------------------------------- */
 
 static void
 update_protocols_window (void)
 {
+  GtkWidget *window;
+  GtkTreeView *gv;
   GtkListStore *gs = NULL;
 
-  /* retrieve view and model (store) */
-  GtkTreeView *gv = GTK_TREE_VIEW (glade_xml_get_widget (xml, "prot_clist"));
-  if (!gv || !GTK_WIDGET_VISIBLE (gv))
-    return;			/* error or hidden */
-  gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
-  if (!gs)
+  window = glade_xml_get_widget (xml, "protocols_window");
+  gv = retrieve_treeview(window);
+  if (!gv)
     {
-      create_protocols_table (glade_xml_get_widget (xml, "protocols_window"), gv);
-      gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
-      if (!gs)
-        return;			/* error */
+      /* register gv */
+      gv = gv = GTK_TREE_VIEW (glade_xml_get_widget (xml, "prot_clist"));
+      if (!gv)
+        {
+          g_critical("can't find prot_clist");
+          return;
+        }
+      register_treeview(window, gv);
     }
-
-  update_protocols_table(gs, protocol_summary_stack());
-
+  
+  update_protocols_table(window, protocol_summary_stack());
 }				/* update_protocols_window */
 
 void
@@ -645,8 +643,7 @@ on_delete_protocol_window (GtkWidget * wdg, GdkEvent * evt, gpointer ud)
 void
 on_protocols_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
-  GtkWidget *protocols_window =
-    glade_xml_get_widget (xml, "protocols_window");
+  GtkWidget *protocols_window = glade_xml_get_widget (xml, "protocols_window");
   if (!protocols_window)
     return;
   if (gtk_check_menu_item_get_active (menuitem))
@@ -656,23 +653,19 @@ on_protocols_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
       update_protocols_window ();
     }
   else
-    gtk_widget_hide (protocols_window);
-}				/* on_protocols_check_activate */
-
-
-/* Displays the protocols window when the legend is double clicked */
-gboolean
-on_prot_table_button_press_event (GtkWidget * widget,
-				  GdkEventButton * event, gpointer user_data)
-{
-  if (GDK_2BUTTON_PRESS == event->type)
     {
-      toggle_protocols_window ();
-      return TRUE;
+      /* retrieve view and model (store) */
+      GtkListStore *gs;
+      GtkTreeView *gv = GTK_TREE_VIEW(glade_xml_get_widget(xml, "prot_clist"));
+      if (gv)
+        {
+          gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
+          if (gs)
+             protocols_table_clear(gs);
+        }
+      gtk_widget_hide (protocols_window);
     }
-
-  return FALSE;
-}				/* on_prot_table_button_press_event */
+}				/* on_protocols_check_activate */
 
 /* opens a protocol detail window when the user clicks a proto row */
 gboolean
@@ -698,7 +691,7 @@ on_prot_list_select_row (GtkTreeView * gv, gboolean arg1, gpointer user_data)
   if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (gs), &it, gpath))
     return FALSE;		/* path not found */
 
-  gtk_tree_model_get (GTK_TREE_MODEL (gs), &it, 5, &protocol, -1);
+  gtk_tree_model_get (GTK_TREE_MODEL (gs), &it, PROTO_COLUMN_N, &protocol, -1);
   create_prot_info_window (protocol);
 
   return TRUE;
@@ -730,7 +723,10 @@ node_info_window_compare(gconstpointer a, gconstpointer b)
 guint
 update_info_windows (void)
 {
-  if (status == PAUSE)
+  enum status_t status;
+
+  status = get_capture_status();
+  if (status != PLAY && status != STOP)
     return TRUE;
 
   gettimeofday (&now, NULL);
@@ -738,6 +734,7 @@ update_info_windows (void)
   update_protocols_window ();
   update_stats_info_windows ();
   update_prot_info_windows ();
+  nodes_wnd_update();
 
   return TRUE;			/* Keep on calling this function */
 
@@ -761,64 +758,19 @@ on_node_info_delete_event (GtkWidget * node_info, GdkEvent * evt,
 }				/* on_node_info_delete_event */
 
 
-void
-node_info_window_create(const node_id_t * node_id)
-{
-  GtkWidget *window;
-  GladeXML *xml_info_window;
-  GList *list_item;
-
-  /* If there is already a window, we don't need to create it again */
-  list_item = g_list_find_custom (stats_info_windows, 
-                                  node_id, node_info_window_compare);
-  if (!list_item)
-    {
-      /* not present, create */
-      xml_info_window =
-	glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "node_info", NULL);
-      if (!xml_info_window)
-	{
-	  g_error (_("We could not load the interface! (%s)"),
-		   GLADEDIR "/" ETHERAPE_GLADE_FILE);
-	  return;
-	}
-      glade_xml_signal_autoconnect (xml_info_window);
-      window = glade_xml_get_widget (xml_info_window, "node_info");
-      gtk_widget_show (window);
-
-      /* register the widgets in the window */
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_name_label");
-      register_glade_widget(xml_info_window, G_OBJECT (window), 
-			      "node_info_numeric_name_label");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_average");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_average_in");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_average_out");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_accumulated");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_accumulated_in");
-      register_glade_widget(xml_info_window, G_OBJECT (window), "node_info_accumulated_out");
-
-      g_object_unref (xml_info_window);
-      g_object_set_data (G_OBJECT (window), "node_id",
-			 g_memdup(node_id, sizeof(node_id_t)));
-      stats_info_windows =
-	g_list_prepend (stats_info_windows, window);
-    }
-  else
-    window = list_item->data;
-  update_node_info_window (window);
-}				/* node_info_window_create */
-
-
 void
 update_stats_info_windows (void)
 {
   GList *list_item;
   GtkWidget *window;
+  enum status_t status;
   struct timeval diff;
   static struct timeval last_time = {
     0, 0
   };
 
+  status = get_capture_status();
+
   diff = substract_times (now, last_time);
   /* Update info windows at most twice a second */
   if (pref.refresh_period < 500)
@@ -843,10 +795,7 @@ update_stats_info_windows (void)
           if (g_object_get_data (G_OBJECT (window), "node_id"))
             {
               /* is a node info window */
-              if (pref.new_infodlg)
-                update_node_protocols_window (window);
-              else
-                update_node_info_window (window);
+              update_node_protocols_window (window);
             }
           else
             update_link_info_window (window);
@@ -857,45 +806,6 @@ update_stats_info_windows (void)
   last_time = now;
 }				/* update_stats_info_windows */
 
-static void
-update_node_info_window (GtkWidget *window)
-{
-  const node_id_t *node_id;
-  const node_t *node;
-
-  node_id = g_object_get_data (G_OBJECT (window), "node_id");
-  node = nodes_catalog_find(node_id);
-  if (!node)
-    {
-      update_gtklabel(window, "node_info_numeric_name_label", _("Node timed out"));
-      update_gtklabel(window, "node_info_average", "X");
-      update_gtklabel(window, "node_info_average_in", "X");
-      update_gtklabel(window, "node_info_average_out", "X");
-      update_gtklabel(window, "node_info_accumulated", "X");
-      update_gtklabel(window, "node_info_accumulated_in", "X");
-      update_gtklabel(window, "node_info_accumulated_out", "X");
-      gtk_widget_queue_resize (GTK_WIDGET (window));
-      return;
-    }
-
-  gtk_window_set_title (GTK_WINDOW (window), node->name->str);
-  
-  update_gtklabel(window, "node_info_name_label", node->name->str);
-  update_gtklabel(window, "node_info_numeric_name_label", node->numeric_name->str);
-  update_gtklabel(window, "node_info_average",
-		      traffic_to_str (node->node_stats.stats.average, TRUE));
-  update_gtklabel(window, "node_info_average_in",
-		      traffic_to_str (node->node_stats.stats_in.average, TRUE));
-  update_gtklabel(window, "node_info_average_out",
-		      traffic_to_str (node->node_stats.stats_out.average, TRUE));
-  update_gtklabel(window, "node_info_accumulated",
-		      traffic_to_str (node->node_stats.stats.accumulated, FALSE));
-  update_gtklabel(window, "node_info_accumulated_in",
-		      traffic_to_str (node->node_stats.stats_in.accumulated, FALSE));
-  update_gtklabel(window, "node_info_accumulated_out",
-		      traffic_to_str (node->node_stats.stats_out.accumulated, FALSE));
-  gtk_widget_queue_resize (GTK_WIDGET (window));
-}				/* update_node_info_window */
 
 /****************************************************************************
  *
@@ -910,11 +820,11 @@ stats_info_create(const gchar *idkey, gpointer key)
   GtkTreeView *gv;
 
   xml_info_window =
-    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "node_proto_info", NULL);
+    glade_xml_new (pref.glade_file, "node_proto_info", NULL);
   if (!xml_info_window)
     {
       g_error (_("We could not load the interface! (%s)"),
-               GLADEDIR "/" ETHERAPE_GLADE_FILE);
+               pref.glade_file);
       return NULL;
     }
   glade_xml_signal_autoconnect (xml_info_window);
@@ -922,23 +832,30 @@ stats_info_create(const gchar *idkey, gpointer key)
   gtk_widget_show (window);
 
   /* register the widgets in the window */
+  register_glade_widget(xml_info_window, G_OBJECT (window), "src_label");
+  register_glade_widget(xml_info_window, G_OBJECT (window), "dst_label");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_name");
   register_glade_widget(xml_info_window, G_OBJECT (window), 
                           "node_iproto_numeric_name");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_name_b");
   register_glade_widget(xml_info_window, G_OBJECT (window), 
                           "node_iproto_numeric_name_b");
+  register_glade_widget(xml_info_window, G_OBJECT (window), "total_label");
+  register_glade_widget(xml_info_window, G_OBJECT (window), "inbound_label");
+  register_glade_widget(xml_info_window, G_OBJECT (window), "outbound_label");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_avg");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_avg_in");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_avg_out");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_accum");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_accum_in");
   register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_accum_out");
-  register_glade_widget(xml_info_window, G_OBJECT (window), "node_iproto_proto");
+
+  /* get and register the tree view */
+  gv = GTK_TREE_VIEW (glade_xml_get_widget (xml_info_window, "node_iproto_proto"));
+  register_treeview(window, gv);
 
   /* create columns of proto list */
-  gv = GTK_TREE_VIEW (glade_xml_get_widget (xml_info_window, "node_iproto_proto"));
-  create_protocols_table (window, gv);
+  create_protocols_table (window);
 
   g_object_unref (xml_info_window);
   
@@ -953,36 +870,49 @@ stats_info_create(const gchar *idkey, gpointer key)
 
 
 static void
-stats_info_update(GtkWidget *window, GtkListStore *gs, const traffic_stats_t *stats)
+stats_info_update(GtkWidget *window, const traffic_stats_t *stats, 
+                  gboolean totals_only)
 {
   if (!stats)
     {
       update_gtklabel(window, "node_iproto_avg", "X");
-      update_gtklabel(window, "node_iproto_avg_in", "X");
-      update_gtklabel(window, "node_iproto_avg_out", "X");
       update_gtklabel(window, "node_iproto_accum", "X");
-      update_gtklabel(window, "node_iproto_accum_in", "X");
-      update_gtklabel(window, "node_iproto_accum_out", "X");
-      gtk_list_store_clear(gs);
+      if (!totals_only)
+        {
+          update_gtklabel(window, "node_iproto_avg_in", "X");
+          update_gtklabel(window, "node_iproto_avg_out", "X");
+          update_gtklabel(window, "node_iproto_accum_in", "X");
+          update_gtklabel(window, "node_iproto_accum_out", "X");
+        }
+      update_protocols_table(window, NULL);
       gtk_widget_queue_resize (GTK_WIDGET (window));
     }
   else
     {
-      update_gtklabel(window, "node_iproto_avg",
-                          traffic_to_str (stats->stats.average, TRUE));
-      update_gtklabel(window, "node_iproto_avg_in",
-                          traffic_to_str (stats->stats_in.average, TRUE));
-      update_gtklabel(window, "node_iproto_avg_out",
-                          traffic_to_str (stats->stats_out.average, TRUE));
-      update_gtklabel(window, "node_iproto_accum",
-                          traffic_to_str (stats->stats.accumulated, FALSE));
-      update_gtklabel(window, "node_iproto_accum_in",
-                          traffic_to_str (stats->stats_in.accumulated, FALSE));
-      update_gtklabel(window, "node_iproto_accum_out",
-                          traffic_to_str (stats->stats_out.accumulated, FALSE));
-    
+      gchar *str;
+      str = traffic_to_str (stats->stats.average, TRUE);
+      update_gtklabel(window, "node_iproto_avg", str);
+      g_free(str);
+      str = traffic_to_str (stats->stats.accumulated, FALSE);
+      update_gtklabel(window, "node_iproto_accum", str);
+      g_free(str);
+      if (!totals_only)
+        {
+          str = traffic_to_str (stats->stats_in.average, TRUE);
+          update_gtklabel(window, "node_iproto_avg_in", str);
+          g_free(str);
+          str = traffic_to_str (stats->stats_out.average, TRUE);
+          update_gtklabel(window, "node_iproto_avg_out", str);
+          g_free(str);
+          str = traffic_to_str (stats->stats_in.accumulated, FALSE);
+          update_gtklabel(window, "node_iproto_accum_in", str);
+          g_free(str);
+          str = traffic_to_str (stats->stats_out.accumulated, FALSE);
+          update_gtklabel(window, "node_iproto_accum_out", str);
+          g_free(str);
+        }
       /* update protocol table */
-      update_protocols_table(gs, &stats->stats_protos);
+      update_protocols_table(window, &stats->stats_protos);
     }
 }
 
@@ -1009,7 +939,7 @@ node_protocols_window_create(const node_id_t * node_id)
   else
     window = list_item->data;
   update_node_protocols_window (window);
-}				/* node_info_window_create */
+}				/* node_protocols_window_create */
 
 static void
 update_node_protocols_window (GtkWidget *window)
@@ -1019,21 +949,13 @@ update_node_protocols_window (GtkWidget *window)
   const node_id_t *node_id;
   const node_t *node;
 
-  /* retrieve view and model (store) */
-  gv = GTK_TREE_VIEW (g_object_get_data (G_OBJECT (window), "node_iproto_proto"));
-  if (!gv)
-    return;			/* error or hidden */
-  gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
-  if (!gs)
-     return;			/* error */
-
   node_id = g_object_get_data (G_OBJECT (window), "node_id");
   node = nodes_catalog_find(node_id);
   if (!node)
     {
       /* node expired, clear stats */
       update_gtklabel(window, "node_iproto_numeric_name", _("Node timed out"));
-      stats_info_update(window, gs, NULL);
+      stats_info_update(window, NULL, FALSE);
       return;
     }
 
@@ -1042,7 +964,7 @@ update_node_protocols_window (GtkWidget *window)
   update_gtklabel(window, "node_iproto_name", node->name->str);
   update_gtklabel(window, "node_iproto_numeric_name", node->numeric_name->str);
   
-  stats_info_update(window, gs, &node->node_stats);
+  stats_info_update(window, &node->node_stats, FALSE);
 }
 
 /****************************************************************************
@@ -1090,18 +1012,18 @@ update_link_info_window (GtkWidget *window)
 {
   GtkListStore *gs;
   GtkTreeView *gv;
+  GtkWidget *widget;
   const link_id_t *link_id;
   const link_t *link;
   const node_t *node;
   gchar *linkname;
 
-  /* retrieve view and model (store) */
-  gv = GTK_TREE_VIEW (g_object_get_data (G_OBJECT (window), "node_iproto_proto"));
-  if (!gv)
-    return;			/* error or hidden */
-  gs = GTK_LIST_STORE (gtk_tree_view_get_model (gv));
-  if (!gs)
-     return;			/* error */
+  /* updates column descriptions */
+  show_widget(window, "src_label");
+  show_widget(window, "dst_label");
+  update_gtklabel(window, "total_label", _("Traffic"));
+  hide_widget(window, "inbound_label");
+  hide_widget(window, "outbound_label");
 
   link_id = g_object_get_data (G_OBJECT (window), "link_id");
   link = links_catalog_find(link_id);
@@ -1110,7 +1032,7 @@ update_link_info_window (GtkWidget *window)
       /* node expired, clear stats */
       update_gtklabel(window, "node_iproto_numeric_name", _("Link timed out"));
       update_gtklabel(window, "node_iproto_numeric_name_b", "");
-      stats_info_update(window, gs, NULL);
+      stats_info_update(window, NULL, TRUE);
       return;
     }
 
@@ -1142,5 +1064,5 @@ update_link_info_window (GtkWidget *window)
     update_gtklabel(window, "node_iproto_numeric_name_b", _("Node timed out"));
   }
 
-  stats_info_update(window, gs, &link->link_stats);
+  stats_info_update(window, &link->link_stats, TRUE);
 }
