@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: info_windows.h,v 1.10 2006/01/11 20:05:50 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,8 @@
 #include <gtk/gtkclist.h>
 #include <gtk/gtkcheckmenuitem.h>
 
-void node_info_window_create(const node_id_t * node_id);
+#include "links.h"
+
 guint update_info_windows (void);
 void node_protocols_window_create(const node_id_t * node_id);
 void link_info_window_create(const link_id_t * link_id);
