@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: main.c,v 1.99 2006/03/25 07:07:44 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,9 +22,10 @@
 #endif
 
 #include <netinet/in.h>
-#include "globals.h"
 #include <signal.h>
+#include <gnome.h>
 #include <libgnomeui/gnome-client.h>
+#include "globals.h"
 #include "ip-cache.h"
 #include "main.h"
 #include "diagram.h"
@@ -33,13 +34,14 @@
 #include "capture.h"
 #include "datastructs.h"
 
+#define ETHERAPE_GLADE_FILE "etherape.glade"	/* glade 3 file */
+
 /***************************************************************************
  *
  * local variables
  *
  **************************************************************************/
 static gboolean quiet = FALSE;
-static GLogLevelFlags debug_mask;
 static void (*oldhandler) (int);
 
 /***************************************************************************
@@ -47,6 +49,7 @@ static void (*oldhandler) (int);
  * internal functions
  *
  **************************************************************************/
+static void free_static_data(void);
 static void set_debug_level (void);
 static void session_die (GnomeClient * client, gpointer client_data);
 static gint
@@ -65,41 +68,41 @@ log_handler (gchar * log_domain,
 int
 main (int argc, char *argv[])
 {
-  gchar *mode_string = NULL;
   GtkWidget *widget;
   GnomeClient *client;
-  gchar *cl_filter = NULL, *cl_interface = NULL, *cl_input_file = NULL;
+  gchar *mode_string = NULL;
+  gchar *cl_filter = NULL;
+  gchar *cl_interface = NULL;
+  gchar *cl_input_file = NULL;
   gboolean cl_numeric = FALSE;
+  glong mdelay = G_MAXLONG;
+  gchar *version;
   poptContext poptcon;
 
   struct poptOption optionsTable[] = {
-    {"mode", 'm', POPT_ARG_STRING, &mode_string, 0,
-     N_("mode of operation"), N_("<ethernet|fddi|ip|tcp>")},
-    {"interface", 'i', POPT_ARG_STRING, &cl_interface, 0,
-     N_("set interface to listen to"), N_("<interface name>")},
-    {"filter", 'f', POPT_ARG_STRING, &cl_filter, 0,
-     N_("set capture filter"), N_("<capture filter>")},
-    {"infile", 'r', POPT_ARG_STRING, &cl_input_file, 0,
-     N_("set input file"), N_("<file name>")},
-    {"numeric", 'n', POPT_ARG_NONE, &cl_numeric, 0,
-     N_("don't convert addresses to names"), NULL},
     {"diagram-only", 'd', POPT_ARG_NONE, &(pref.diagram_only), 0,
      N_("don't display any node text identification"), NULL},
-    {"no-fade", 'F', POPT_ARG_NONE, &(pref.nofade), 0,
-     N_("do not fade old links"), NULL},
-    {"stationary", 's', POPT_ARG_NONE, &(pref.stationary), 0,
-     N_("don't move nodes around"), NULL},
-    {"node_limit", 'l', POPT_ARG_INT, &(pref.node_limit), 0,
+    {"replay-file", 'r', POPT_ARG_STRING, &cl_input_file, 0,
+     N_("replay packets from file"), N_("<file to replay>")},
+    {"filter", 'f', POPT_ARG_STRING, &cl_filter, 0,
+     N_("set capture filter"), N_("<capture filter>")},
+    {"interface", 'i', POPT_ARG_STRING, &cl_interface, 0,
+     N_("set interface to listen to"), N_("<interface name>")},
+    {"stationary", 's', POPT_ARG_NONE, &(pref.stationary), 0,  
+     N_("don't move nodes around"), NULL}, 
+    {"node-limit", 'l', POPT_ARG_INT, &(pref.node_limit), 0,
      N_("limits nodes displayed"), N_("<number of nodes>")},
+    {"mode", 'm', POPT_ARG_STRING, &mode_string, 0,
+     N_("mode of operation"), N_("<ethernet|fddi|ip|tcp>")},
+    {"numeric", 'n', POPT_ARG_NONE, &cl_numeric, 0,
+     N_("don't convert addresses to names"), NULL},
     {"quiet", 'q', POPT_ARG_NONE, &quiet, 0,
-     N_("Don't show warnings"), NULL},
-    {"node-color", 'N', POPT_ARG_STRING, &(pref.node_color), 0,
-     N_("set the node color"), N_("color")},
-    {"text-color", 'T', POPT_ARG_STRING, &(pref.text_color), 0,
-     N_("set the text color"), N_("color")},
-    {"zero-delay", 'z', POPT_ARG_NONE, &(pref.zero_delay), 0,
-     N_("zero delay for reading capture files [cli only]"), NULL},
-
+     N_("Disable informational messages"), NULL},
+    {"max-delay", 0, POPT_ARG_LONG, &mdelay,  0,
+     N_("maximum packet delay in ms for reading capture files [cli only]"),
+      N_("<delay>")},
+    {"glade-file", 0, POPT_ARG_STRING, &(pref.glade_file), 0,
+     N_("uses the named libglade file for widgets"), N_("<glade file>")},
 
     POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
   };
@@ -115,30 +118,25 @@ main (int argc, char *argv[])
   if (!getenv ("GNOME_DESKTOP_ICON"))
     putenv ("GNOME_DESKTOP_ICON=" PIXMAPS_DIR "/etherape.png");
 
-
+#ifdef PACKAGE_SCM_REV
   /* We initiate the application and read command line options */
-  gnome_program_init ("EtherApe", VERSION, LIBGNOMEUI_MODULE, argc, argv,
+  version = g_strdup_printf("%s (hg id %s)", VERSION, 
+                            (*PACKAGE_SCM_REV) ? PACKAGE_SCM_REV : 
+                              _("-unknown-"));
+#else
+  version = g_strdup(VERSION);
+#endif
+  gnome_program_init ("EtherApe", version, 
+                      LIBGNOMEUI_MODULE, argc, argv,
 		      GNOME_PARAM_POPT_TABLE, optionsTable, GNOME_PARAM_NONE);
+  g_free(version);
 
 
   /* We obtain application parameters 
    * First, absolute defaults
    * Second, values saved in the config file
    * Third, whatever given in the command line */
-
-  /* Absolute defaults */
-  pref.name_res = TRUE;
-  pref.mode = IP;
-  pref.filter = NULL;
-  status = STOP;
-  pref.refresh_period = 800;	/* ms */
-
-  /* TODO Besides the fact that this probably makes little sense nowadays
-   * (at least for node color) it probably leads to a segfault
-   * See how it is done for filter, for instance */
-  pref.node_color = g_strdup ("brown");
-  pref.text_color = g_strdup ("yellow");
-  pref.node_limit = -1;
+  init_config(&pref);
 
   set_debug_level ();
 
@@ -185,30 +183,48 @@ main (int argc, char *argv[])
 	pref.mode = IP;
       else if (strstr (mode_string, "tcp"))
 	pref.mode = TCP;
-      else if (strstr (mode_string, "udp"))
-	pref.mode = UDP;
       else
 	g_warning (_
 		   ("Unrecognized mode. Do etherape --help for a list of modes"));
     }
 
+  if (mdelay >= 0)
+    {
+      if (mdelay < G_MAXLONG)
+        {
+          pref.max_delay = mdelay;
+          g_message("Maximum delay set to %lu ms", pref.max_delay);
+        }
+    }
+  else
+      g_message("Invalid maximum delay %ld, ignored", mdelay);
+  
   /* Glade */
 
   glade_gnome_init ();
+  glade_require("gnome");
+  glade_require("canvas");
 
-  xml = glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, NULL, NULL);
+  if (!pref.glade_file)
+    pref.glade_file = GLADEDIR "/" ETHERAPE_GLADE_FILE;
+
+  xml = glade_xml_new (pref.glade_file, NULL, NULL);
   if (!xml)
     {
-      g_error (_("We could not load the interface! (%s)"),
-	       GLADEDIR "/" ETHERAPE_GLADE_FILE);
+      g_error (_("We could not load glade interface file! (%s)"),
+	       pref.glade_file);
       return 1;
     }
   glade_xml_signal_autoconnect (xml);
 
   app1 = glade_xml_get_widget (xml, "app1");
+  statusbar = GTK_STATUSBAR(glade_xml_get_widget (xml, "statusbar1"));
 
+  /* prepare decoders */
+  services_init();
+  
   /* Sets controls to the values of variables and connects signals */
-  init_diagram ();
+  init_diagram (xml);
 
   /* Session handling */
   client = gnome_master_client ();
@@ -218,7 +234,6 @@ main (int argc, char *argv[])
 		    GTK_SIGNAL_FUNC (session_die), NULL);
   gtk_widget_show (app1);
 
-
   /* 
    * Signal handling
    * Catch SIGINT and SIGTERM and, if we get either of them, clean up
@@ -233,18 +248,13 @@ main (int argc, char *argv[])
     signal (SIGHUP, oldhandler);
 #endif
 
-
   /* With this we force an update of the diagram every x ms 
    * Data in the diagram is updated, and then the canvas redraws itself when
    * the gtk loop is idle. If the CPU can't handle the set refresh_period,
    * then it will just do a best effort */
 
   widget = glade_xml_get_widget (xml, "canvas1");
-  diagram_timeout = g_timeout_add_full (G_PRIORITY_DEFAULT,
-					pref.refresh_period,
-					(GtkFunction) update_diagram,
-					widget,
-					(GDestroyNotify) destroying_timeout);
+  destroying_idle (widget);
 
   /* This other timeout makes sure that the info windows are updated */
   g_timeout_add (500, (GtkFunction) update_info_windows, NULL);
@@ -253,54 +263,55 @@ main (int argc, char *argv[])
   g_timeout_add (10000, (GtkFunction) ipcache_tick, NULL);
 
   init_menus ();
-
+  
   gui_start_capture ();
 
-
   /* MAIN LOOP */
   gtk_main ();
 
-  protohash_clear();
+  free_static_data();
   return 0;
 }				/* main */
 
-
-
-
-
+/* releases all static and cached data. Called just before exiting. Obviously 
+ * it's not stricly needed, since the memory will be returned to the OS anyway,
+ * but makes finding memory leaks much easier. */
+static void free_static_data(void)
+{
+  protohash_clear();
+  ipcache_clear();
+  services_clear();
+}
 
 static void
 set_debug_level (void)
 {
   const gchar *env_debug;
-  env_debug = g_getenv ("DEBUG");
+  env_debug = g_getenv("APE_DEBUG");
 
-  debug_mask = (G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_INFO));
+  pref.debug_mask = (G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_INFO));
 
   if (env_debug)
     {
-      if (!strcmp (env_debug, "INFO"))
-	debug_mask = (G_LOG_LEVEL_MASK & ~G_LOG_LEVEL_DEBUG);
-      else if (!strcmp (env_debug, "DEBUG"))
-	debug_mask = G_LOG_LEVEL_MASK;
+      if (!g_ascii_strcasecmp(env_debug, "INFO"))
+	pref.debug_mask = (G_LOG_LEVEL_MASK & ~G_LOG_LEVEL_DEBUG);
+      else if (!g_ascii_strcasecmp(env_debug, "DEBUG"))
+	pref.debug_mask = G_LOG_LEVEL_MASK;
     }
 
   if (quiet)
-    debug_mask = 0;
-
-  /* if someone requested debug infos, enable the relevant flag */
-  pref.is_debug = (debug_mask & G_LOG_LEVEL_DEBUG) ? TRUE : FALSE;
+    pref.debug_mask = 0;
 
   g_log_set_handler (NULL, G_LOG_LEVEL_MASK, (GLogFunc) log_handler, NULL);
-  g_my_debug ("debug_mask %d", debug_mask);
+  g_my_debug ("debug_mask %d", pref.debug_mask);
 }
 
 static void
 log_handler (gchar * log_domain,
 	     GLogLevelFlags mask, const gchar * message, gpointer user_data)
 {
-  if (mask & debug_mask)
-    g_log_default_handler (NULL, mask, message, user_data);
+  if (mask & pref.debug_mask)
+    g_log_default_handler ("EtherApe", mask, message, user_data);
 }
 
 /* the gnome session manager may call this function */
@@ -322,6 +333,8 @@ save_session (GnomeClient * client, gint phase, GnomeSaveStyle save_style,
 
   /* allocate 0-filled, so it will be NULL-terminated */
   argv = g_malloc0 (sizeof (gchar *) * 4);
+  g_assert(argv);
+
   argc = 1;
 
   argv[0] = client_data;
@@ -338,7 +351,6 @@ save_session (GnomeClient * client, gint phase, GnomeSaveStyle save_style,
 
   gnome_client_set_clone_command (client, argc, argv);
   gnome_client_set_restart_command (client, argc, argv);
-
   return TRUE;
 }				/* save_session */
 
@@ -352,5 +364,6 @@ void
 cleanup (int signum)
 {
   cleanup_capture ();
+  free_static_data();
   gtk_exit (0);
 }
