@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: menus.c,v 1.31 2006/05/13 20:31:05 bchiara Exp $
+ * $Id$
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include "menus.h"
 #include <string.h>
-#include <libgnomeui/gnome-file-entry.h>
 #include <gtk/gtkcontainer.h>
+#include <glib.h>
+#include "menus.h"
 #include "util.h"
 #include "diagram.h"
 #include "info_windows.h"
@@ -31,33 +31,6 @@ static gboolean in_start_capture = FALSE;
 
 static void set_active_interface (void);
 
-
-#if 0
-static GnomeUIInfo help_submenu[] = {
-  GNOMEUIINFO_HELP ("etherape"),
-  GNOMEUIINFO_END
-};
-
-/* This is here just as an example in case I need it again */
-void
-on_file1_activate (GtkMenuItem * menuitem, gpointer user_data)
-{
-  GtkWidget *messagebox;
-  GladeXML *xml_messagebox;
-  xml_messagebox =
-    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "messagebox1", NULL);
-  messagebox = glade_xml_get_widget (xml_messagebox, "messagebox1");
-  if (!xml)
-    {
-      g_error (_("We could not load the interface! (%s)"),
-	       GLADEDIR "/" ETHERAPE_GLADE_FILE);
-      return;
-    }
-  gtk_widget_show (messagebox);
-  gtk_object_unref (GTK_OBJECT (xml_messagebox));
-}
-#endif
-
 void
 init_menus (void)
 {
@@ -116,73 +89,38 @@ init_menus (void)
 void
 on_open_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
-  static GtkWidget *open_file_dialog = NULL;
-  if (!open_file_dialog)
-    open_file_dialog = glade_xml_get_widget (xml, "open_file_dialog");
-  gtk_widget_show (open_file_dialog);
-}				/* on_open_activate */
+  GtkWidget *dialog;
 
-void
-on_file_cancel_button_clicked (GtkButton * button, gpointer user_data)
-{
-  GtkWidget *open_file_dialog;
-  open_file_dialog = glade_xml_get_widget (xml, "open_file_dialog");
-  gtk_widget_hide (open_file_dialog);
-}				/* on_file_cancel_button_clicked */
+  if (!gui_stop_capture ())
+    return;
 
-/* Sets up the new input capture file */
-void
-on_file_combo_entry_changed (GtkEditable * editable, gpointer user_data)
-{
-  gchar *str;
-  str = gtk_editable_get_chars (editable, 0, -1);
+  dialog = gtk_file_chooser_dialog_new ("Open Capture File",
+				      NULL,
+				      GTK_FILE_CHOOSER_ACTION_OPEN,
+				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+				      NULL);
   if (pref.input_file)
-    g_free (pref.input_file);
+    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), pref.input_file);
 
-  pref.input_file = g_strdup (str);
-  g_free (str);
-
-}				/* on_file_combo_entry_changed */
-
-void
-on_file_ok_button_clicked (GtkButton * button, gpointer user_data)
-{
-  GtkWidget *widget;
-  gchar *str = NULL;
-
-  if (status != STOP)
-    if (!gui_stop_capture ())
-      return;
-
-  if (pref.interface)
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
     {
+      GtkRecentManager *manager;
+      manager = gtk_recent_manager_get_default ();
+      gtk_recent_manager_add_item (manager, gtk_file_chooser_get_uri(GTK_FILE_CHOOSER (dialog)));
+
+      g_free(pref.input_file);
+      pref.input_file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+      gtk_widget_destroy (dialog);
+
       g_free (pref.interface);
       pref.interface = NULL;
+      
+      gui_start_capture ();
     }
-
-  widget = glade_xml_get_widget (xml, "file_combo_entry");
-  str = gtk_editable_get_chars (GTK_EDITABLE (widget), 0, -1);
-
-  widget = glade_xml_get_widget (xml, "open_file_dialog");
-  gtk_widget_hide (widget);
-
-
-  if (pref.input_file)
-    g_free (pref.input_file);
-  pref.input_file = g_strdup (str);
-  if (str)
-    g_free (str);
-
-  gnome_entry_append_history (GNOME_ENTRY
-			      (gnome_file_entry_gnome_entry
-			       (GNOME_FILE_ENTRY
-				(glade_xml_get_widget (xml, "fileentry")))),
-			      TRUE, pref.input_file);
-
-  gui_start_capture ();
-
-}				/* on_file_ok_button_clicked */
-
+  else
+    gtk_widget_destroy (dialog);
+}				/* on_open_activate */
 
 /* Capture menu */
 
@@ -199,9 +137,8 @@ on_interface_radio_activate (gchar * gui_device)
 				 * of interface look change from
 				 * start_capture */
 
-  if (status != STOP)
-    if (!gui_stop_capture ())
-      return;
+  if (!gui_stop_capture ())
+     return;
 
   if (pref.input_file)
     g_free (pref.input_file);
@@ -243,8 +180,6 @@ on_mode_radio_activate (GtkMenuItem * menuitem, gpointer user_data)
     new_mode = IP;
   else if (!strcmp ("tcp_radio", menuname))
     new_mode = TCP;
-  else if (!strcmp ("udp_radio", menuname))
-    new_mode = UDP;
   else
     {
       g_my_critical ("Unsopported mode in on_mode_radio_activate");
@@ -284,9 +219,9 @@ on_mode_radio_activate (GtkMenuItem * menuitem, gpointer user_data)
     default:
       return;
     }
-  if (status != STOP)
-    if (!gui_stop_capture ())
-      return;
+
+  if (!gui_stop_capture ())
+     return;
   pref.mode = new_mode;
   g_my_info (_("Mode set to %s in GUI"), (gchar *) menuitem);
   gui_start_capture ();
@@ -313,7 +248,6 @@ on_stop_menuitem_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
   g_my_debug ("on_stop_menuitem_activate called");
   gui_stop_capture ();
-
 }				/* on_stop_menuitem_activate */
 
 
@@ -335,7 +269,7 @@ on_toolbar_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
   GtkWidget *widget;
 
-  widget = glade_xml_get_widget (xml, "dock_toolbar");
+  widget = glade_xml_get_widget (xml, "handlebox_toolbar");
   if (menuitem->active)
     gtk_widget_show (widget);
   else
@@ -347,7 +281,7 @@ on_legend_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
   GtkWidget *widget;
 
-  widget = glade_xml_get_widget (xml, "dock_legend");
+  widget = glade_xml_get_widget (xml, "handlebox_legend");
   if (menuitem->active)
     gtk_widget_show (widget);
   else
@@ -357,12 +291,10 @@ on_legend_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 void
 on_status_bar_check_activate (GtkCheckMenuItem * menuitem, gpointer user_data)
 {
-  GtkWidget *widget;
-  widget = glade_xml_get_widget (xml, "appbar1");
   if (menuitem->active)
-    gtk_widget_show (widget);
+    gtk_widget_show (GTK_WIDGET(statusbar));
   else
-    gtk_widget_hide (widget);
+    gtk_widget_hide (GTK_WIDGET(statusbar));
 }				/* on_status_bar_check_activate */
 
 
@@ -375,25 +307,20 @@ void
 on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
 {
   GtkWidget *about;
-  GladeXML *xml_about;
-
-  xml_about =
-    glade_xml_new (GLADEDIR "/" ETHERAPE_GLADE_FILE, "about2", NULL);
-  if (!xml_about)
-    {
-      g_error (_("We could not load the interface! (%s)"),
-	       GLADEDIR "/" ETHERAPE_GLADE_FILE);
-      return;
-    }
-  about = glade_xml_get_widget (xml_about, "about2");
-  g_object_unref (xml_about);
+  gchar *msg;
+  about = glade_xml_get_widget (xml, "about1");
 
+  gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
+#ifdef PACKAGE_SCM_REV
+  msg = g_strdup_printf("HG revision: %s", 
+                        (*PACKAGE_SCM_REV) ? PACKAGE_SCM_REV : _("-unknown-"));
+  gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), msg);
+  g_free(msg);
+#endif
   gtk_widget_show (about);
-
 }				/* on_about1_activate */
 
 
-
 /* Helper functions */
 
 /* Sets up the GUI to reflect changes and calls start_capture() */
@@ -404,20 +331,20 @@ gui_start_capture (void)
   gchar *errorbuf = NULL;
   GString *status_string = NULL;
 
-  if ((status == PAUSE) && end_of_file)
+  if (get_capture_status() == CAP_EOF)
     if (!gui_stop_capture ())
       return;
 
-  if (status == STOP)
+  if (get_capture_status() == STOP)
     if ((errorbuf = init_capture ()) != NULL)
       {
 	fatal_error_dialog (errorbuf);
 	return;
       }
 
-  if (status == PLAY)
+  if (get_capture_status() == PLAY)
     {
-      g_warning (_("Status not STOP or PAUSE at gui_start_capture"));
+      g_warning (_("Status already PLAY at gui_start_capture"));
       return;
     }
 
@@ -507,9 +434,6 @@ gui_start_capture (void)
     case TCP:
       widget = glade_xml_get_widget (xml, "tcp_radio");
       break;
-    case UDP:
-      widget = glade_xml_get_widget (xml, "udp_radio");
-      break;
     default:
       g_warning (_("Invalid mode: %d"), pref.mode);
       return;
@@ -519,8 +443,7 @@ gui_start_capture (void)
   /* Set the interface in GUI */
   set_active_interface ();
 
-  /* Sets the appbar */
-
+  /* Sets the statusbar */
   status_string = g_string_new (_("Reading data from "));
 
   if (pref.input_file)
@@ -547,15 +470,12 @@ gui_start_capture (void)
     case TCP:
       g_string_append (status_string, _(" in TCP mode"));
       break;
-    case UDP:
-      g_string_append (status_string, _(" in UDP mode"));
-      break;
     default:
       g_critical (_("Invalid mode: %d"), pref.mode);
       return;
     }
 
-  set_appbar_status (status_string->str);
+  set_statusbar_msg (status_string->str);
   g_string_free (status_string, TRUE);
 
   in_start_capture = FALSE;
@@ -563,20 +483,12 @@ gui_start_capture (void)
   g_my_info (_("Diagram started"));
 }				/* gui_start_capture */
 
-
-
 void
 gui_pause_capture (void)
 {
   GtkWidget *widget;
   GString *status_string = NULL;
 
-  if (status == PAUSE)
-    {
-      g_warning (_("Status not PLAY at gui_pause_capture"));
-      return;
-    }
-
   /*
    * Make sure the data in the info windows is updated
    * so that it is consistent
@@ -599,19 +511,43 @@ gui_pause_capture (void)
   widget = glade_xml_get_widget (xml, "pause_menuitem");
   gtk_widget_set_sensitive (widget, FALSE);
 
-
-  /* Sets the appbar */
-
-  status_string = g_string_new (_("Paused"));
-
-  set_appbar_status (status_string->str);
-  g_string_free (status_string, TRUE);
+  set_statusbar_msg (_("Paused"));
 
   g_my_info (_("Diagram paused"));
-
+  dump_stats(0);
 }				/* gui_pause_capture */
 
 
+/* reached eof on a file replay */
+void gui_eof_capture(void)
+{
+  GtkWidget *widget;
+  GString *status_string = NULL;
+
+  if (get_capture_status() == STOP)
+    return;
+
+  widget = glade_xml_get_widget (xml, "start_button");
+  gtk_widget_set_sensitive (widget, TRUE);
+  widget = glade_xml_get_widget (xml, "start_menuitem");
+  gtk_widget_set_sensitive (widget, TRUE);
+/*  widget = glade_xml_get_widget (xml, "stop_button");
+  gtk_widget_set_sensitive (widget, FALSE);
+  widget = glade_xml_get_widget (xml, "stop_menuitem");
+  gtk_widget_set_sensitive (widget, FALSE);*/
+  widget = glade_xml_get_widget (xml, "pause_button");
+  gtk_widget_set_sensitive (widget, FALSE);
+  widget = glade_xml_get_widget (xml, "pause_menuitem");
+  gtk_widget_set_sensitive (widget, FALSE);
+
+  /* Sets the statusbar */
+  status_string = g_string_new ("");
+  g_string_printf(status_string, _("Replay from file '%s' completed."), pref.input_file);
+  set_statusbar_msg (status_string->str);
+  g_string_free (status_string, TRUE);
+}				/* gui_stop_capture */
+
+
 /* Sets up the GUI to reflect changes and calls stop_capture() */
 gboolean
 gui_stop_capture (void)
@@ -619,6 +555,9 @@ gui_stop_capture (void)
   GtkWidget *widget;
   GString *status_string = NULL;
 
+  stop_requested = FALSE;
+  if (get_capture_status() == STOP)
+    return TRUE;
 
   /*
    * gui_stop_capture needs to call update_diagram in order to
@@ -627,13 +566,9 @@ gui_stop_capture (void)
    * might end up being called below another update_diagram. We can't
    * allow two simultaneous calls, so we fail
    */
-
   if (already_updating)
-    return FALSE;
-
-  if (status == STOP)
     {
-      g_warning (_("Status not PLAY or PAUSE at gui_stop_capture"));
+      stop_requested = TRUE;
       return FALSE;
     }
 
@@ -659,11 +594,7 @@ gui_stop_capture (void)
   widget = glade_xml_get_widget (xml, "canvas1");
   update_diagram (widget);
 
-  /* Now update info windows */
-  update_info_windows ();
-
-  /* Sets the appbar */
-
+  /* Sets the statusbar */
   status_string = g_string_new (_("Ready to capture from "));
 
   if (pref.input_file)
@@ -673,11 +604,11 @@ gui_stop_capture (void)
   else
     g_string_append (status_string, _("default interface"));
 
-  set_appbar_status (status_string->str);
+  set_statusbar_msg (status_string->str);
   g_string_free (status_string, TRUE);
 
   g_my_info (_("Diagram stopped"));
-
+  dump_stats(0);
   return TRUE;
 }				/* gui_stop_capture */
 
@@ -724,31 +655,4 @@ set_active_interface ()
       menu_items = menu_items->next;
     }
 
-#if 0
-  GList *entry_items = NULL;
-  gboolean is_new = TRUE, normal_case = TRUE;
-
-  /* The combo fills up even if there are duplicate values. I think
-   * this is a bug in gnome and might be fixed in the future, but
-   * right now I'll just make sure the history is fine and reload that*/
-
-  /* TODO There really should be a better way of getting all values from
-   * the drop down list than this casting mess :-( */
-
-  /* The file entry introduces a copy of the file name at the end of the
-   * list if the browse button has been used. There goes another hack to
-   * work around it */
-  entry_items = GTK_LIST (GTK_COMBO (gentry)->list)->children;
-  while (entry_items)
-    {
-      gchar *history_value;
-      history_value = GTK_LABEL (GTK_BIN (entry_items->data)->child)->label;
-      if (!strcmp (history_value, str) && !is_fileentry
-	  && (entry_items->next != NULL))
-	is_new = FALSE;
-      normal_case = !(is_fileentry && !(entry_items->next));
-      entry_items = g_list_next (entry_items);
-    }
-#endif
-
 }				/* set_active_interface */
