@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: menus.h,v 1.11 2006/05/13 20:31:05 bchiara Exp $
+ * $Id$
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,10 @@
 #include <config.h>
 #endif
 
-#include <libgnomeui/gnome-popup-menu.h>
-#include <libgnomeui/gnome-messagebox.h>
 #include <gtk/gtk.h>
 #include "globals.h"
 
 void on_open_activate (GtkMenuItem * menuitem, gpointer user_data);
-void on_file_cancel_button_clicked (GtkButton * button, gpointer user_data);
-void on_file_combo_entry_changed (GtkEditable * editable, gpointer user_data);
-void on_file_ok_button_clicked (GtkButton * button, gpointer user_data);
 
 void on_interface_radio_activate (gchar * gui_device);
 void on_mode_radio_activate (GtkMenuItem * menuitem, gpointer user_data);
@@ -46,3 +41,12 @@ void on_status_bar_check_activate (GtkCheckMenuItem * menuitem,
 				   gpointer user_data);
 
 void on_about1_activate (GtkMenuItem * menuitem, gpointer user_data);
+
+void init_menus (void);
+void fatal_error_dialog (const gchar * message);
+void gui_start_capture (void);
+void gui_pause_capture (void);
+void gui_eof_capture (void);
+gboolean gui_stop_capture (void);	/* gui_stop_capture might fail. For instance,
+					 * it can't run if diagram_update is running */
+
