@@ -1,6 +1,6 @@
 /* EtherApe
  * Copyright (C) 2001 Juan Toledo
- * $Id: names.c,v 1.44 2006/04/29 07:58:35 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,13 +17,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <gnome.h>
 #include <netinet/in.h>
+#include "globals.h"
+#include "names.h"
 #include "dns.h"
 #include "eth_resolv.h"
 #include "names_netbios.h"
-#include "names.h"
 #include "protocols.h"
+#include "util.h"
 
 typedef struct
 {
@@ -36,7 +37,7 @@ typedef struct
   struct
   {
     guint8 level;       /* current decoder level */
-    gchar **tokens;     /* array of decoder names */
+    const packet_protos_t *tokens;     /* array of decoder names */
     protostack_t *protos;  /* protocol list */
   } decoder;
 }
@@ -92,15 +93,30 @@ static prot_function_t prot_functions_table[KNOWN_PROTS + 1] = {
   {"NETBIOS-DGM", get_nbdgm_name}
 };
 
-static void add_name (gchar * numeric, gchar * resolved, gboolean solved,
-                      const node_id_t *node_id, const name_add_t *nt);
+static void missing_data_msg(const name_add_t *nt, const char *pr)
+{
+  if (pr)
+    g_warning
+    (_("not enough captured data, terminating protocol decode for '%s' (level %d)"),
+    pr, nt->decoder.level);
+  else
+    g_warning
+    (_("not enough captured data, terminating protocol decode at level %d"),
+    nt->decoder.level);
+    
+}
+
+static void add_name (const gchar * numeric, const gchar * resolved, 
+                      gboolean solved, const node_id_t *node_id, 
+                      const name_add_t *nt);
 static void decode_next(name_add_t *nt);
 
 void
 get_packet_names (protostack_t *pstk,
 		  const guint8 * packet,
 		  guint16 size,
-		  const gchar * prot_stack, packet_direction direction)
+		  const packet_protos_t * prot_stack, 
+                  packet_direction direction)
 {
   name_add_t nt;
 
@@ -117,13 +133,11 @@ get_packet_names (protostack_t *pstk,
    * We initialize as 0 because decode_next() preincrements
    */
   nt.decoder.level = 0;
-  nt.decoder.tokens = g_strsplit (prot_stack, "/", 0);
+  nt.decoder.tokens = prot_stack;
   nt.decoder.protos = pstk;
 
   /* starts decoding */
   decode_next(&nt);
-
-  g_strfreev (nt.decoder.tokens);
 }				/* get_packet_names */
 
 /* increments the level and calls the next name decoding function, if any */
@@ -144,22 +158,19 @@ static void decode_next(name_add_t *nt)
     }
 
   g_assert(nt);
-  g_assert(nt->decoder.tokens[nt->decoder.level]); /* current level must be valid */
+  g_assert(nt->decoder.tokens->protonames[nt->decoder.level]); /* current level must be valid */
 
   nt->decoder.level++;
 
-  if (!nt->decoder.tokens[nt->decoder.level])
-    return; /* no more decoders, exit */
+  if (!nt->decoder.tokens->protonames[nt->decoder.level])
+    return; /* no more levels, exit */
 
-  next_func = g_tree_lookup (prot_functions, nt->decoder.tokens[nt->decoder.level]);
+  next_func = g_tree_lookup (prot_functions, nt->decoder.tokens->protonames[nt->decoder.level]);
   if (next_func)
     {
       /* before calling the next decoder, we check for size overflow */
       if (nt->packet_size <= nt->offset)
-        {
-          g_critical(_("captured data insufficient, aborting protocol decode"));
           return;
-        }
   
       next_func->function (nt);
     }
@@ -208,7 +219,7 @@ static void fill_node_id(node_id_t *node_id, apemode_t apemode, const name_add_t
   if (nt->packet_size < nt->offset + disp + sz ||
      nt->packet_size < nt->offset + portdisp + sz)
     {
-      g_error(_("captured data insufficient, aborting fill_node_id"));
+      missing_data_msg(nt, NULL);
       return;
     }
 
@@ -261,16 +272,16 @@ get_linux_sll_name (name_add_t *nt)
   decode_next(nt);
 }				/* get_linux_sll_name */
 
-static void
-get_eth_name (name_add_t *nt)
+/* common handling for ethernet-like data */
+static void eth_name_common(apemode_t ethmode, name_add_t *nt)
 {
-  gchar *numeric = NULL, *solved = NULL;
+  const gchar *numeric, *solved;
   gboolean found_in_ethers = FALSE;
 
   if (nt->dir == INBOUND)
-    fill_node_id(&nt->node_id, ETHERNET, nt, 0, 0);
+    fill_node_id(&nt->node_id, ethmode, nt, ethmode, 0);
   else
-    fill_node_id(&nt->node_id, ETHERNET, nt, 6, 0);
+    fill_node_id(&nt->node_id, ethmode, nt, ethmode+6, 0);
 
   numeric = ether_to_str (nt->node_id.addr.eth);
   solved = get_ether_name (nt->node_id.addr.eth);
@@ -294,76 +305,24 @@ get_eth_name (name_add_t *nt)
   nt->offset += 14;
 
   decode_next(nt);
+}
+
+static void
+get_eth_name (name_add_t *nt)
+{
+  eth_name_common(ETHERNET, nt);
 }				/* get_eth_name */
 
 static void
 get_ieee802_name (name_add_t *nt)
 {
-  gchar *numeric = NULL, *solved = NULL;
-  gboolean found_in_ethers = FALSE;
-
-  if (nt->dir == INBOUND)
-    fill_node_id(&nt->node_id, IEEE802, nt, 2, 0);
-  else
-    fill_node_id(&nt->node_id, IEEE802, nt, 8, 0);
-
-  numeric = ether_to_str (nt->node_id.addr.i802);
-  solved = get_ether_name (nt->node_id.addr.i802);
-
-  /* get_ether_name will return an ethernet address with
-   * the first three numbers substituted with the manufacter
-   * if it cannot find an /etc/ethers entry. If it is so,
-   * then the last 8 characters (for example ab:cd:ef) will
-   * be the same, and we will note that the name hasn't
-   * been solved */
-
-  if (numeric && solved)
-    found_in_ethers = strcmp (numeric + strlen (numeric) - 8,
-			      solved + strlen (solved) - 8);
-
-  if (found_in_ethers)
-    add_name (numeric, solved, TRUE, &nt->node_id, nt);
-  else
-    add_name (numeric, solved, FALSE, &nt->node_id, nt);
-
-  nt->offset += 14;
-
-  decode_next(nt);
+  eth_name_common(IEEE802, nt);
 }				/* get_ieee802_name */
 
 static void
 get_fddi_name (name_add_t *nt)
 {
-  gchar *numeric = NULL, *solved = NULL;
-  gboolean found_in_ethers = FALSE;
-
-  if (nt->dir == INBOUND)
-    fill_node_id(&nt->node_id, FDDI, nt, 1, 0);
-  else
-    fill_node_id(&nt->node_id, FDDI, nt, 7, 0);
-
-  numeric = ether_to_str (nt->node_id.addr.fddi);
-  solved = get_ether_name (nt->node_id.addr.fddi);
-
-  /* get_ether_name will return an ethernet address with
-   * the first three numbers substituted with the manufacter
-   * if it cannot find an /etc/ethers entry. If it is so,
-   * then the last 8 characters (for example ab:cd:ef) will
-   * be the same, and we will note that the name hasn't
-   * been solved */
-
-  if (numeric && solved)
-    found_in_ethers = strcmp (numeric + strlen (numeric) - 8,
-			      solved + strlen (solved) - 8);
-
-  if (found_in_ethers)
-    add_name (numeric, solved, TRUE, &nt->node_id, nt);
-  else
-    add_name (numeric, solved, FALSE, &nt->node_id, nt);
-
-  nt->offset += 13;
-
-  decode_next(nt);
+  eth_name_common(FDDI, nt);
 }				/* get_fddi_name */
 
 /* LLC is the only supported FDDI link layer type */
@@ -399,7 +358,7 @@ get_arp_name (name_add_t *nt)
 
   if (nt->packet_size <= nt->offset + 4)
     {
-      g_critical(_("captured data insufficient, aborting get_arp_name"));
+      missing_data_msg(nt, "ARP");
       return;
     }
 
@@ -410,7 +369,7 @@ get_arp_name (name_add_t *nt)
 
   if (nt->packet_size <= nt->offset + 7)
     {
-      g_critical(_("captured data insufficient, aborting get_arp_name"));
+      missing_data_msg(nt, "ARP");
       return;
     }
 
@@ -419,12 +378,9 @@ get_arp_name (name_add_t *nt)
 
   fill_node_id(&nt->node_id, IP, nt, 8 + hardware_len, 0);
 
-  if (pref.mode == ETHERNET)
-    add_name (ip_to_str (nt->node_id.addr.ip4), dns_lookup (pntohl (nt->node_id.addr.ip4), 
-              FALSE), TRUE, &nt->node_id, nt);
-  else
-    add_name (ip_to_str (nt->node_id.addr.ip4), dns_lookup (pntohl (nt->node_id.addr.ip4), 
-              TRUE), TRUE, &nt->node_id, nt);
+  add_name (ip_to_str (nt->node_id.addr.ip4), 
+            dns_lookup (pntohl (nt->node_id.addr.ip4), TRUE), 
+            TRUE, &nt->node_id, nt);
 
   /* ARP doesn't carry any other protocol on top, so we return 
    * directly */
@@ -440,19 +396,15 @@ get_ip_name (name_add_t *nt)
     fill_node_id(&nt->node_id, IP, nt, 12, 0);
 
   if (!pref.name_res)
-    add_name (ip_to_str (nt->node_id.addr.ip4), ip_to_str (nt->node_id.addr.ip4), FALSE, &nt->node_id, nt);
+    add_name (ip_to_str (nt->node_id.addr.ip4), 
+              ip_to_str (nt->node_id.addr.ip4), FALSE, &nt->node_id, nt);
   else
     {
-      if (pref.mode == ETHERNET)
-	add_name (ip_to_str(nt->node_id.addr.ip4), dns_lookup(pntohl (nt->node_id.addr.ip4), FALSE), TRUE, &nt->node_id, nt);
-      else
-	add_name (ip_to_str(nt->node_id.addr.ip4), dns_lookup(pntohl (nt->node_id.addr.ip4), TRUE), TRUE, &nt->node_id, nt);
+      add_name (ip_to_str(nt->node_id.addr.ip4), 
+                dns_lookup(pntohl (nt->node_id.addr.ip4), TRUE), 
+                TRUE, &nt->node_id, nt);
     }
 
-  /* TODO I don't like the fact that the gdk_input for dns.c is not
-   * called in this function, because it means that it can't be used 
-   * as a library */
-
   nt->offset += 20;
 
   decode_next(nt);
@@ -471,33 +423,37 @@ get_tcp_name (name_add_t *nt)
 {
   guint8 th_off_x2;
   guint8 tcp_len;
-  GString *numeric_name, *resolved_name;
 
-  if (nt->dir == OUTBOUND)
-      fill_node_id(&nt->node_id, TCP, nt, -8, 0);
-  else
-      fill_node_id(&nt->node_id, TCP, nt, -4, 2);
+  /* tcp names are useful only if someone uses them ... */
+  if (pref.mode == TCP)
+    {
+      gchar *numeric_name, *resolved_name;
+      if (nt->dir == OUTBOUND)
+          fill_node_id(&nt->node_id, TCP, nt, -8, 0);
+      else
+          fill_node_id(&nt->node_id, TCP, nt, -4, 2);
 
-  numeric_name = g_string_new("");
-  g_string_printf(numeric_name, "%s:%d",ip_to_str(nt->node_id.addr.tcp4.host),
-                                *(guint16 *) (nt->node_id.addr.tcp4.port));
+      numeric_name = g_strdup_printf("%s:%d",
+                                     ip_to_str(nt->node_id.addr.tcp4.host),
+                                     *(guint16 *) (nt->node_id.addr.tcp4.port));
+
+      resolved_name = g_strdup_printf("%s:%s",
+                                      dns_lookup (
+                                                  pntohl (nt->node_id.addr.tcp4.host), 
+                                                  TRUE),
+                                      get_tcp_port
+                                         (*(guint16 *)(nt->node_id.addr.tcp4.port))
+                                     );
+
+      add_name (numeric_name, resolved_name, TRUE, &nt->node_id, nt);
+
+      g_free (numeric_name);
+      g_free (resolved_name);
+    }
   
-  resolved_name = g_string_new (dns_lookup (pntohl (nt->node_id.addr.tcp4.host), TRUE));
-  resolved_name = g_string_append_c (resolved_name, ':');
-  resolved_name = g_string_append (resolved_name,
-				   get_tcp_port (*(guint16 *)
-						 (nt->node_id.addr.tcp4.port)));
-
-  add_name (numeric_name->str, resolved_name->str, TRUE, &nt->node_id, nt);
-
-  g_string_free (numeric_name, TRUE);
-  numeric_name = NULL;
-  g_string_free (resolved_name, TRUE);
-  resolved_name = NULL;
-
   if (nt->packet_size <= nt->offset + 14)
     {
-      g_critical(_("captured data insufficient, aborting get_tcp_name"));
+      missing_data_msg(nt, "TCP");
       return;
     }
 
@@ -540,11 +496,11 @@ get_ipxsap_name (name_add_t *nt)
   for (curpos = nt->offset + 4; curpos < nt->packet_size ; ++curpos)
     {
       if ( ! *((gchar *) (nt->p + curpos)) )
-        break; // found termination
+        break; /* found termination */
     }
   if (curpos >= nt->packet_size)
     {
-      g_critical(_("captured data insufficient, aborting get_ipxsap_name"));
+      missing_data_msg(nt, "IPXSAP");
       return;
     }
     
@@ -587,7 +543,7 @@ get_nbss_name (name_add_t *nt)
                                      * know what it might mean in the ethereal implementation */
       if (nt->packet_size <= nt->offset + 2)
         {
-          g_critical(_("captured data insufficient, aborting get_nbss_name (on decoding netbios lenght)"));
+          missing_data_msg(nt, "NBSS");
           return;
         }
       length = pntohs ((nt->p + nt->offset + 2));
@@ -595,7 +551,7 @@ get_nbss_name (name_add_t *nt)
       nt->offset += 2;
       if (nt->packet_size <= nt->offset + length)
         {
-          g_critical(_("captured data insufficient, aborting get_nbss_name (on decoding name)"));
+          missing_data_msg(nt, "NBSS");
           return;
         }
 
@@ -685,25 +641,25 @@ get_nbdgm_name (name_add_t *nt)
       add_name (numeric_name, name, TRUE, &nt->node_id, nt);
       g_free (numeric_name);
     }
-
-
 }				/* get_nbdgm_name */
 
 
-
 static void
-add_name (gchar * numeric_name, gchar * resolved_name, gboolean solved, 
-          const node_id_t *node_id, const name_add_t *nt)
+add_name (const gchar * numeric_name, const gchar * resolved_name, 
+          gboolean solved, const node_id_t *node_id, const name_add_t *nt)
 {
   protocol_t *protocol = NULL;
   GList *name_item = NULL;
   name_t *name = NULL;
   name_t key;
 
-  /* Find the protocol entry */
-  protocol = protocol_stack_find(nt->decoder.protos,
+  /* Find the protocol entry
+     note: protocol_stack_find returns a const ptr, but this function
+     modifies it. At this point, is safe
+   */
+  protocol = (protocol_t *)protocol_stack_find(nt->decoder.protos,
                                  nt->decoder.level,
-			         nt->decoder.tokens[nt->decoder.level]);
+			         nt->decoder.tokens->protonames[nt->decoder.level]);
 
   key.node_id = *node_id;
   
