@@ -1,6 +1,6 @@
 /* Etherape
  * Copyright (C) 2000 Juan Toledo
- * $Id: names.h,v 1.17 2006/01/04 19:48:09 bchiara Exp $
+ * $Id$
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,4 +23,5 @@
 void get_packet_names (protostack_t *pstk,
 		       const guint8 * packet,
 		       guint16 size,
-		       const gchar * prot_stack, packet_direction direction);
+		       const packet_protos_t * prot_stack, 
+                       packet_direction direction);
