@@ -1,6 +1,6 @@
 /* resolv.c
  * Routines for network object lookup
- * $Id: resolv.c,v 1.21 2006/08/13 05:21:52 bchiara Exp $
+ * $Id$
  *
  * Originally written by Laurent Deniel <deniel@worldnet.fr>
  * Adapted to etherape by Juan Toledo <toledo@users.sourceforge.net>
@@ -124,7 +124,6 @@ typedef struct _ether
 }
 ether_t;
 
-static hashname_t *udp_port_table[HASHPORTSIZE];
 static hashname_t *tcp_port_table[HASHPORTSIZE];
 static hashmanuf_t *manuf_table[HASHMANUFSIZE];
 static hashether_t *eth_table[HASHETHSIZE];
@@ -135,8 +134,6 @@ static int eth_resolution_initialized = 0;
  *  Global variables (original impl. changed them in GUI sections)
  */
 
-static int g_resolving_actif = 1;	/* routines are active by default */
-
 static gchar *g_ethers_path = EPATH_ETHERS;
 static gchar *g_pethers_path = NULL;	/* "$HOME"/EPATH_PERSONAL_ETHERS    */
 static gchar *g_manuf_path = EPATH_MANUF;	/* may only be changed before the   */
@@ -159,10 +156,6 @@ serv_name_lookup (u_int port, u_int proto)
 
   switch (proto)
     {
-    case IPPROTO_UDP:
-      table = udp_port_table;
-      serv_proto = "udp";
-      break;
     case IPPROTO_TCP:
       table = tcp_port_table;
       serv_proto = "tcp";
@@ -181,6 +174,7 @@ serv_name_lookup (u_int port, u_int proto)
     {
       tp = table[i & (HASHPORTSIZE - 1)] =
 	(hashname_t *) g_malloc (sizeof (hashname_t));
+      g_assert(tp);
     }
   else
     {
@@ -193,6 +187,7 @@ serv_name_lookup (u_int port, u_int proto)
 	  if (tp->next == NULL)
 	    {
 	      tp->next = (hashname_t *) g_malloc (sizeof (hashname_t));
+              g_assert(tp->next);
 	      tp = tp->next;
 	      break;
 	    }
@@ -443,6 +438,7 @@ add_manuf_name (u_char * addr, char * name)
     {
       tp = table[((int) addr[2]) & (HASHMANUFSIZE - 1)] =
 	(hashmanuf_t *) g_malloc (sizeof (hashmanuf_t));
+      g_assert(tp);
     }
   else
     {
@@ -451,6 +447,7 @@ add_manuf_name (u_char * addr, char * name)
 	  if (tp->next == NULL)
 	    {
 	      tp->next = (hashmanuf_t *) g_malloc (sizeof (hashmanuf_t));
+              g_assert(tp->next);
 	      tp = tp->next;
 	      break;
 	    }
@@ -535,6 +532,7 @@ eth_name_lookup (const u_char * addr)
     {
       tp = table[(i ^ j) & (HASHETHSIZE - 1)] =
 	(hashether_t *) g_malloc (sizeof (hashether_t));
+      g_assert(tp);
     }
   else
     {
@@ -547,6 +545,7 @@ eth_name_lookup (const u_char * addr)
 	  if (tp->next == NULL)
 	    {
 	      tp->next = (hashether_t *) g_malloc (sizeof (hashether_t));
+              g_assert(tp->next);
 	      tp = tp->next;
 	      break;
 	    }
@@ -582,70 +581,16 @@ eth_name_lookup (const u_char * addr)
 
 }				/* eth_name_lookup */
 
-
-extern char *
-get_udp_port (u_int port)
-{
-  static gchar str[3][MAXNAMELEN];
-  static gchar *cur;
-
-  if (!g_resolving_actif)
-    {
-      if (cur == &str[0][0])
-	{
-	  cur = &str[1][0];
-	}
-      else if (cur == &str[1][0])
-	{
-	  cur = &str[2][0];
-	}
-      else
-	{
-	  cur = &str[0][0];
-	}
-      snprintf (cur, MAXNAMELEN, "%d", port);
-      return cur;
-    }
-
-  return serv_name_lookup (port, IPPROTO_UDP);
-
-}				/* get_udp_port */
-
 extern char *
 get_tcp_port (u_int port)
 {
-  static gchar str[3][MAXNAMELEN];
-  static gchar *cur;
-
-  if (!g_resolving_actif)
-    {
-      if (cur == &str[0][0])
-	{
-	  cur = &str[1][0];
-	}
-      else if (cur == &str[1][0])
-	{
-	  cur = &str[2][0];
-	}
-      else
-	{
-	  cur = &str[0][0];
-	}
-      snprintf (cur, MAXNAMELEN, "%d", port);
-      return cur;
-    }
-
   return serv_name_lookup (port, IPPROTO_TCP);
-
 }				/* get_tcp_port */
 
 
 extern char *
 get_ether_name (const u_char * addr)
 {
-  if (!g_resolving_actif)
-    return ether_to_str ((guint8 *) addr);
-
   if (!eth_resolution_initialized)
     {
       initialize_ethers ();
@@ -656,37 +601,3 @@ get_ether_name (const u_char * addr)
 
 }				/* get_ether_name */
 
-extern char *
-get_manuf_name (u_char * addr)
-{
-  static gchar str[3][MAXMANUFLEN];
-  static gchar *cur;
-  hashmanuf_t *manufp;
-
-  if (g_resolving_actif && !eth_resolution_initialized)
-    {
-      initialize_ethers ();
-      eth_resolution_initialized = 1;
-    }
-
-  if (!g_resolving_actif || ((manufp = manuf_name_lookup (addr)) == NULL))
-    {
-      if (cur == &str[0][0])
-	{
-	  cur = &str[1][0];
-	}
-      else if (cur == &str[1][0])
-	{
-	  cur = &str[2][0];
-	}
-      else
-	{
-	  cur = &str[0][0];
-	}
-      snprintf (cur, MAXMANUFLEN, "%02x:%02x:%02x", addr[0], addr[1], addr[2]);
-      return cur;
-    }
-
-  return manufp->name;
-
-}				/* get_manuf_name */
