@@ -5,7 +5,7 @@
 /* util.c
  * Utility routines
  *
- * $Id: util.c,v 1.16 2006/05/15 18:48:44 bchiara Exp $
+ * $Id$
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -33,8 +33,6 @@
 #endif
 
 #include <glib.h>
-#include <gnome.h>
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -182,3 +180,115 @@ safe_strncat (char *dst, const char *src, size_t maxlen)
   dst[maxlen - 1] = '\0';
   return dst;
 }
+
+/* Next three functions copied directly from ethereal packet.c
+ * by Gerald Combs */
+
+/* Output has to be copied elsewhere */
+/* TODO should I dump this funtion now that I have dns_lookup? */
+const gchar *
+ip_to_str (const guint8 * ad)
+{
+  static gchar str[3][16];
+  static gchar *cur;
+  gchar *p;
+  int i;
+  guint32 octet;
+  guint32 digit;
+
+  if (cur == &str[0][0])
+    {
+      cur = &str[1][0];
+    }
+  else if (cur == &str[1][0])
+    {
+      cur = &str[2][0];
+    }
+  else
+    {
+      cur = &str[0][0];
+    }
+  p = &cur[16];
+  *--p = '\0';
+  i = 3;
+  for (;;)
+    {
+      octet = ad[i];
+      *--p = (octet % 10) + '0';
+      octet /= 10;
+      digit = octet % 10;
+      octet /= 10;
+      if (digit != 0 || octet != 0)
+	*--p = digit + '0';
+      if (octet != 0)
+	*--p = octet + '0';
+      if (i == 0)
+	break;
+      *--p = '.';
+      i--;
+    }
+  return p;
+}				/* ip_to_str */
+
+/* (toledo) This function I copied from capture.c of ethereal it was
+ * without comments, but I believe it keeps three different
+ * strings conversions in memory so as to try to make sure that
+ * the conversions made will be valid in memory for a longer
+ * period of time */
+
+/* Places char punct in the string as the hex-digit separator.
+ * If punct is '\0', no punctuation is applied (and thus
+ * the resulting string is 5 bytes shorter)
+ */
+
+const gchar *
+ether_to_str_punct (const guint8 * ad, char punct)
+{
+  static gchar str[3][18];
+  static gchar *cur;
+  gchar *p;
+  int i;
+  guint32 octet;
+  static const gchar hex_digits[16] = "0123456789abcdef";
+
+  if (cur == &str[0][0])
+    {
+      cur = &str[1][0];
+    }
+  else if (cur == &str[1][0])
+    {
+      cur = &str[2][0];
+    }
+  else
+    {
+      cur = &str[0][0];
+    }
+  p = &cur[18];
+  *--p = '\0';
+  i = 5;
+  for (;;)
+    {
+      octet = ad[i];
+      *--p = hex_digits[octet & 0xF];
+      octet >>= 4;
+      *--p = hex_digits[octet & 0xF];
+      if (i == 0)
+	break;
+      if (punct)
+	*--p = punct;
+      i--;
+    }
+  return p;
+}				/* ether_to_str_punct */
+
+
+
+/* Wrapper for the most common case of asking
+ * for a string using a colon as the hex-digit separator.
+ */
+const gchar *
+ether_to_str (const guint8 * ad)
+{
+  return ether_to_str_punct (ad, ':');
+}				/* ether_to_str */
+
