@@ -1,7 +1,7 @@
 /* util.h
  * Utility definitions
  *
- * $Id: util.h,v 1.6 2006/04/29 07:58:35 bchiara Exp $
+ * $Id$
  *
  * Ethereal - Network traffic analyzer
  * By Gerald Combs <gerald@zing.org>
@@ -26,6 +26,8 @@
 #ifndef __UTIL_H__
 #define __UTIL_H__
 
+#include "globals.h"
+
 #ifdef __cplusplus
 extern "C"
 {
@@ -44,9 +46,13 @@ extern "C"
   char *safe_strncpy (char *dst, const char *src, size_t maxlen);
   char *safe_strncat (char *dst, const char *src, size_t maxlen);
 
+  /* utility functions */
+  const gchar *ip_to_str (const guint8 * ad);
+  const gchar *ether_to_str (const guint8 * ad);
+  const gchar *ether_to_str_punct (const guint8 * ad, char punct);
+  
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
 
-
 #endif				/* __UTIL_H__ */
